/*
	AppWindow.cc

	Will Eighteen, 2000, 2006: will@opaline.demon.co.uk
*/

#include "AppWindow.h"
#include <Application.h>
#include <InfoDialogManager.h>
#include <MessageDialogManager.h>
#include <WorkingDialogManager.h>
#include <FileSelect.h>
#include <MenuBar.h>
#include <CmdList.h>
#include <DMenuCmd.h>
#include <X11/cursorfont.h>
#include "ExitCmd.h"
#include "ViewWaveCmd.h"
#include "WaveInfoCmd.h"
#include "TemplateInfoCmd.h"
#include <GraphFltCmd.h>
#include <GraphIntCmd.h>
#include <Xm/Form.h>
#include "EditWaveCmd.h"
#include <FileAccess.h>
#include "SelectFilesCmd.h"
#include "AsciiToFloatCmd.h"
#include "AsciiToHetroCmd.h"
#include "WavToRawCmd.h"
#include "RawToWavCmd.h"
#include "WavToIntCmd.h"
#include "IntToFloatCmd.h"
#include "IntToWavCmd.h"
#include "FloatToAsciiCmd.h"
#include "FloatToIntCmd.h"
#include "HetroToAsciiCmd.h"
#include "SplitWaveCmd.h"
#include "NewTemplateCmd.h"
#include "ViewTemplateCmd.h"
#include "DeleteCmd.h"
#include "ViewTemplateCmd.h"
#include "ModifyArrayCmd.h"
#include "SysCtrlPanel.h"
#include "WaveCtrlCmd.h"
#include "SaveStateCmd.h"
#include "LoadStateCmd.h"
#include "MixCmd.h"
#include "NewMixCmd.h"
#include "DeletePakCmd.h"
#include "CsoundCmd.h"
#include "MixwaveUtils.h"
#include "CreateSpatialTemplateCmd.h"
#include "NewSpatialTemplateCmd.h"
#include "ShapeTemplateCmd.h"

// subclasses used to maintain user-definable resources
#include "FPDigitsCmd.h"
#include "NumDigitsCmd.h"
#include "ViewWindowSizeCmd.h"
#include "MixBufferSizeCmd.h"
#include "WaveDefaultsCmd.h"
#include "ConvertDefaultsIntCmd.h"
#include "ConvertDefaultsFloatCmd.h"
#include "FltRangeCmd.h"
#include "IntRangeCmd.h"
#include "ChoicesCmd.h"
#include "TemplateSizeCmd.h"
#include "CsoundSampleSizeCmd.h"
#include "CsoundKRateCmd.h"
#include "MixDefaultsCmd.h"
#include "WorkingDirCmd.h"

// help
#include "AboutCmd.h"
#include "EditHelpCmd.h"
#include "FileHelpCmd.h"
#include "MixerHelpCmd.h"
#include "OptionsHelpCmd.h"
#include "SpatialHelpCmd.h"
#include "ViewerHelpCmd.h"


// define edit operations
enum OPTYPE { REMOVE, INSERT };


List<WavePak> *wavePak = new List<WavePak>;
List<TemplatePak> *templatePak = new List<TemplatePak>;
List<MixPak> *mixPak = new List<MixPak>;

OpResources *mixwaveResources = new OpResources ( "MixwaveResources" );


AppWindow::AppWindow ( const char *name ) : MenuWindow ( name )
{
	_sysCtrlPanel = NULL;
}


AppWindow::~AppWindow()
{
	delete _sysCtrlPanel;
}


Widget AppWindow::createWorkArea ( Widget parent )
{
	Widget form = XtCreateWidget ( "workArea", xmFormWidgetClass, parent,
									NULL, 0 );

	_sysCtrlPanel = new SysCtrlPanel ( "systemControl", form );
	_sysCtrlPanel->manage();
	return ( form );
}


void AppWindow::createMenuPanes()
{
	Cmd *newWave = new FileSelect ( "Wave", TRUE, &AppWindow::newWaveCallback,
					&AppWindow::fileSelectHelpCallback,
					&AppWindow::fileSelectCancelledCallback, ( void * ) this );

	Cmd *newMix = new NewMixCmd ( "Mix", FALSE );

	Cmd *loadWave = new FileSelect ( "Wave", TRUE, &AppWindow::loadWaveCallback,
					&AppWindow::fileSelectHelpCallback,
					&AppWindow::fileSelectCancelledCallback, ( void * ) this );

	Cmd *loadTemplate = new SelectFilesCmd ( "Template", TRUE, &AppWindow::loadTemplateCallback,
						"load", "sync_wave", False, ( void * ) this );

	Cmd *viewWave = new ViewWaveCmd ( "Wave", FALSE );
	Cmd *viewTemplate = new ViewTemplateCmd ( "Template", FALSE );
	Cmd *saveState = new SaveStateCmd ( "Save", FALSE );	// no save with no info
	Cmd *loadState = new LoadStateCmd ( "Load", TRUE );
	Cmd *waveInfo = new WaveInfoCmd ( "Wave", FALSE );
	Cmd *templateInfo = new TemplateInfoCmd ( "Template", FALSE );

	Cmd *mix = new FileSelect ( "Mixdown", FALSE, &AppWindow::mixdownCallback,
					&AppWindow::fileSelectHelpCallback,
					&AppWindow::fileSelectCancelledCallback, ( void * ) this );

	Cmd *exit = new ExitCmd ( "Exit", TRUE );	// subclassed from 'QuitCmd'

	Cmd *copy = new EditWaveCmd ( "Copy", FALSE, INSERT );
	Cmd *cut = new EditWaveCmd ( "Cut", FALSE, REMOVE );

	Cmd *dataChange = new ModifyArrayCmd ( "Shift/scale", FALSE,
						mixwaveResources->numDigits(),
						mixwaveResources->fpDigits() );

	Cmd *shapeTemplate = new ShapeTemplateCmd ( "Shape template", FALSE,
							mixwaveResources->numDigits(),
							mixwaveResources->fpDigits() );

	Cmd *split = new FileSelect ( "Split stereo", TRUE, &AppWindow::splitWaveCallback,
					&AppWindow::fileSelectHelpCallback,
					&AppWindow::fileSelectCancelledCallback, ( void * ) this );

	Cmd *wavToRaw = new SelectFilesCmd ( "Wave to Raw", TRUE, &AppWindow::wavToRawCallback,
						"select_source", "select_target", True, ( void * ) this );
	Cmd *rawToWav = new SelectFilesCmd ( "Raw to Wave", TRUE, &AppWindow::rawToWavCallback,
						"select_source", "select_target", True, ( void * ) this );
	Cmd *intToWav = new SelectFilesCmd ( "Int to Wave", TRUE, &AppWindow::intToWavCallback,
						"select_source", "select_target", True, ( void * ) this );
	Cmd *wavToInt = new SelectFilesCmd ( "Wave to Int", TRUE, &AppWindow::wavToIntCallback,
						"select_source", "select_target", True, ( void * ) this );
	Cmd *intToFloat = new SelectFilesCmd ( "Int to Float", TRUE, &AppWindow::intToFloatCallback,
						"select_source", "select_target", True, ( void * ) this );
	Cmd *floatToInt = new SelectFilesCmd ( "Float to Int", TRUE, &AppWindow::floatToIntCallback,
						"select_source", "select_target", True, ( void * ) this );
	Cmd *asciiToFloat = new SelectFilesCmd ( "Ascii to Float", TRUE, &AppWindow::asciiToFloatCallback,
						"select_source", "select_target", True, ( void * ) this );
	Cmd *floatToAscii = new SelectFilesCmd ( "Float to Ascii", TRUE, &AppWindow::floatToAsciiCallback,
						"select_source", "select_target", True, ( void * ) this );
	Cmd *asciiToHetro = new SelectFilesCmd ( "Ascii to Hetro", TRUE, &AppWindow::asciiToHetroCallback,
						"select_source", "select_target", True, ( void * ) this );
	Cmd *hetroToAscii = new SelectFilesCmd ( "Hetro to Ascii", TRUE, &AppWindow::hetroToAsciiCallback,
						"select_source", "select_target", True, ( void * ) this );
	Cmd *intTemplate = new NewTemplateCmd ( "Integer", TRUE, 0 );	// 0=int, 1=float
	Cmd *fltTemplate = new NewTemplateCmd ( "Float", TRUE, 1 );		// 0=int, 1=float

	Cmd *newSpatialTemplate = new NewSpatialTemplateCmd ( "Spatial", TRUE,
														&AppWindow::newSpatialTemplateCallback, ( void  * ) this );

	Cmd *deleteWave = new DeleteCmd ( "Wave", FALSE, 0 );			// 0=wave
	Cmd *deleteTemplate = new DeleteCmd ( "Template", FALSE, 1 );	// 1=template
	Cmd *deletePak = new DeletePakCmd ( "All", FALSE );

	// create list of commands and install in menu pane
	CmdList *cmdListFile = new CmdList ( "File" );
	CmdList *cmdListNew = new CmdList ( "New" );
	CmdList *cmdListLoad = new CmdList ( "Load" );
	CmdList *cmdListState = new CmdList ( "State" );	
	CmdList *cmdListEdit = new CmdList ( "Edit" );
	CmdList *cmdListInfo = new CmdList ( "Info" );
	CmdList *cmdListConvert = new CmdList ( "Convert" );
	CmdList *cmdListCsoundConvert = new CmdList ( "Csound" );
	CmdList *cmdListNewTemplate = new CmdList ( "Template" );
	CmdList *cmdListView = new CmdList ( "View" );
	CmdList *cmdListDelete = new CmdList ( "Delete" );

	cmdListLoad->add ( loadWave );
	cmdListLoad->add ( loadTemplate );

	cmdListDelete->add ( deleteWave );
	cmdListDelete->add ( deleteTemplate );
	cmdListDelete->add ( deletePak );

	cmdListView->add ( viewWave );
	cmdListView->add ( viewTemplate );

	cmdListNew->add ( newWave );
	cmdListNew->add ( cmdListNewTemplate );
	cmdListNew->add ( newMix );

	cmdListNewTemplate->add ( intTemplate );
	cmdListNewTemplate->add ( fltTemplate );

	if ( strlen ( mixwaveResources->csoundCmd() ) != 0 )
		cmdListNewTemplate->add ( newSpatialTemplate );

	cmdListState->add ( loadState );
	cmdListState->add ( saveState );

	cmdListInfo->add ( waveInfo );
	cmdListInfo->add ( templateInfo );

	cmdListFile->add ( cmdListNew );
	cmdListFile->add ( cmdListLoad );
	cmdListFile->add ( cmdListView );
	cmdListFile->add ( cmdListDelete );
	cmdListFile->add ( cmdListState );
	cmdListFile->add ( cmdListInfo );
	cmdListFile->add ( mix );
	cmdListFile->add ( exit );

	cmdListEdit->add ( copy );
	cmdListEdit->add ( cut );
	cmdListEdit->add ( dataChange );
	cmdListEdit->add ( shapeTemplate );
	cmdListEdit->add ( cmdListConvert );

	cmdListConvert->add ( split );		// stereo -> 2x mono
	cmdListConvert->add ( wavToRaw );
	cmdListConvert->add ( rawToWav );
	cmdListConvert->add ( intToWav );
	cmdListConvert->add ( wavToInt );
	cmdListConvert->add ( intToFloat );
	cmdListConvert->add ( floatToInt );
	cmdListConvert->add ( asciiToFloat );
	cmdListConvert->add ( floatToAscii );

	cmdListCsoundConvert->add ( asciiToHetro );
	cmdListCsoundConvert->add ( hetroToAscii );
	cmdListConvert->add ( cmdListCsoundConvert );

	_menuBar->addCommands ( cmdListFile );
	_menuBar->addCommands ( cmdListEdit );

	delete cmdListLoad;
	delete cmdListDelete;
	delete cmdListView;
	delete cmdListNewTemplate;
	delete cmdListNew;
	delete cmdListFile;
	delete cmdListState;
	delete cmdListEdit;
	delete cmdListInfo;
	delete cmdListCsoundConvert;
	delete cmdListConvert;

	Cmd *numDigits = new NumDigitsCmd ( "INT precision" );
	Cmd *fpDigits = new FPDigitsCmd ( "FP precision" );
	Cmd *viewWindowSize = new ViewWindowSizeCmd ( "View size" );
	Cmd *waveDefaults = new WaveDefaultsCmd ( "Wave parameters" );
	Cmd *templateSize = new TemplateSizeCmd ( "Template size" );
	Cmd *mixBufSize = new MixBufferSizeCmd ( "Mix buffer size" );
	Cmd *intConversion = new ConvertDefaultsIntCmd ( "<type> to Int" );
	Cmd *floatConversion = new ConvertDefaultsFloatCmd ( "<type> to Float" );
	Cmd *fltRange = new FltRangeCmd ( "FP range" );
	Cmd *intRange = new IntRangeCmd ( "INT range" );
	Cmd *choices = new ChoicesCmd ( "System control", TRUE );
	Cmd *waveCtrl = new WaveCtrlCmd ( "Wave control", TRUE );
	Cmd *wkgDir = new WorkingDirCmd ( "Directory", TRUE );
	Cmd *mixParams = new MixDefaultsCmd ( "Mix parameters", TRUE );

	Cmd *csoundCommand = new CsoundCmd ( "Csound", TRUE );
	Cmd *csoundSampleSize = new CsoundSampleSizeCmd ( "Wave samples" );
	Cmd *csoundControlRate = new CsoundKRateCmd ( "Control rate" );

	CmdList *cmdListOpts = new CmdList ( "Options" );
	CmdList *cmdListFileOpts = new CmdList ( "File" );
	CmdList *cmdListDisplayOpts = new CmdList ( "Display" );
	CmdList *cmdListMixOpts = new CmdList ( "Mix" );
	CmdList *cmdListConvertOpts = new CmdList ( "Convert" );
	CmdList *cmdListCsoundOpts = new CmdList ( "Csound" );

	cmdListCsoundOpts->add ( csoundCommand );

	if ( strlen ( mixwaveResources->csoundCmd() ) != 0 )
	{
		cmdListCsoundOpts->add ( csoundSampleSize );
		cmdListCsoundOpts->add ( csoundControlRate );
	}

	cmdListConvertOpts->add ( intConversion );
	cmdListConvertOpts->add ( floatConversion );

	cmdListFileOpts->add ( waveDefaults );
	cmdListFileOpts->add ( templateSize );

	cmdListDisplayOpts->add ( numDigits );
	cmdListDisplayOpts->add ( fpDigits );
	cmdListDisplayOpts->add ( viewWindowSize );
	cmdListDisplayOpts->add ( intRange );
	cmdListDisplayOpts->add ( fltRange );

	cmdListMixOpts->add ( mixParams );
	cmdListMixOpts->add ( mixBufSize );

	cmdListOpts->add ( choices );
	cmdListOpts->add ( waveCtrl );
	cmdListOpts->add ( wkgDir );
	cmdListOpts->add ( cmdListMixOpts );
	cmdListOpts->add ( cmdListFileOpts );
	cmdListOpts->add ( cmdListDisplayOpts );
	cmdListOpts->add ( cmdListConvertOpts );
	cmdListOpts->add ( cmdListCsoundOpts );

	_menuBar->addCommands ( cmdListOpts );

	delete cmdListCsoundOpts;
	delete cmdListConvertOpts;
	delete cmdListMixOpts;
	delete cmdListFileOpts;
	delete cmdListDisplayOpts;
	delete cmdListOpts;

	// create command objects for 'Help' menu
	Cmd *about = new AboutCmd ( "About", TRUE );
	Cmd *fileHelp = new FileHelpCmd ( "File menu", TRUE );
	Cmd *editHelp = new EditHelpCmd ("Edit menu", TRUE );
	Cmd *mixerHelp = new MixerHelpCmd ("Mixer", TRUE );
	Cmd *optionsHelp = new OptionsHelpCmd ( "Options menu", TRUE );
	Cmd *spatialHelp = new SpatialHelpCmd ( "Spatialise", TRUE );
	Cmd *viewerHelp = new ViewerHelpCmd ( "Viewer", TRUE );

	// create list of commands and install in menu pane
	CmdList *cmdListHelp = new CmdList ( "Help" );

	cmdListHelp->add ( about );
	cmdListHelp->add ( fileHelp );
	cmdListHelp->add ( editHelp );
	cmdListHelp->add ( optionsHelp );
	cmdListHelp->add ( mixerHelp );
	cmdListHelp->add ( viewerHelp );
	cmdListHelp->add ( spatialHelp );

	_menuBar->addCommands ( cmdListHelp );

	delete cmdListHelp;

	// set up dependencies
	loadWave->addToDeactivationList ( loadState );
	loadTemplate->addToDeactivationList ( loadState );
	newWave->addToDeactivationList ( loadState );
	loadState->addToDeactivationList ( loadState );
	deletePak->addToActivationList ( loadState );
	loadWave->addToActivationList ( saveState );
	loadState->addToActivationList ( saveState );
	newWave->addToActivationList ( saveState );
	deletePak->addToDeactivationList ( saveState );	
	loadWave->addToActivationList ( newMix );
	loadState->addToActivationList ( newMix );
	newWave->addToActivationList ( newMix );
	deletePak->addToDeactivationList ( newMix );
	loadWave->addToActivationList ( mix );
	loadState->addToActivationList (mix );
	newWave->addToActivationList ( mix );
	deletePak->addToDeactivationList ( mix );
	newWave->addToActivationList ( deleteWave );
	newWave->addToActivationList ( deletePak );
	loadWave->addToActivationList ( deleteWave );
	loadWave->addToActivationList ( deletePak );
	loadTemplate->addToActivationList (deleteTemplate );
	loadTemplate->addToActivationList (deletePak );
	loadState->addToActivationList ( deleteWave );
	loadState->addToActivationList ( deleteTemplate );
	loadState->addToActivationList ( deletePak );
	deletePak->addToDeactivationList ( deleteWave );
	deletePak->addToDeactivationList ( deleteTemplate );
	deletePak->addToDeactivationList ( deletePak );
	loadWave->addToActivationList ( waveInfo );
	loadState->addToActivationList (waveInfo );
	newWave->addToActivationList ( waveInfo );
	deletePak->addToDeactivationList ( waveInfo );
	loadTemplate->addToActivationList ( templateInfo );
	intTemplate->addToActivationList ( templateInfo );
	fltTemplate->addToActivationList ( templateInfo );
	loadState->addToActivationList ( templateInfo );
	deletePak->addToDeactivationList ( templateInfo );
	newWave->addToActivationList ( copy );
	loadWave->addToActivationList ( copy );
	loadState->addToActivationList ( copy );
	deletePak->addToDeactivationList ( copy );
	newWave->addToActivationList ( cut );
	loadWave->addToActivationList ( cut );
	loadState->addToActivationList ( cut );
	deletePak->addToDeactivationList ( cut );
	loadWave->addToActivationList ( viewWave );
	newWave->addToActivationList ( viewWave );
	deletePak->addToDeactivationList ( viewWave );
	loadState->addToActivationList ( viewWave );
	intTemplate->addToActivationList ( viewTemplate );
	fltTemplate->addToActivationList ( viewTemplate );
	loadTemplate->addToActivationList ( viewTemplate );
	loadState->addToActivationList ( viewTemplate );
	deletePak->addToDeactivationList ( viewTemplate );
	loadWave->addToActivationList ( dataChange );
	newWave->addToActivationList ( dataChange );
	loadTemplate->addToActivationList ( dataChange );
	intTemplate->addToActivationList ( dataChange );
	fltTemplate->addToActivationList ( dataChange );
	loadState->addToActivationList ( dataChange );
	deletePak->addToDeactivationList ( dataChange );
	loadTemplate->addToActivationList ( shapeTemplate );
	intTemplate->addToActivationList ( shapeTemplate );
	fltTemplate->addToActivationList ( shapeTemplate );
	loadState->addToActivationList ( shapeTemplate );
	deletePak->addToDeactivationList ( shapeTemplate );
}


void AppWindow::newWaveCallback ( void *clientData, char *filename )
{
	if ( filename[strlen(filename) - 1] != '/' )	// no file specified
	{
		AppWindow *obj = ( AppWindow * ) clientData;
		obj->newWave ( filename );
	}
}


void AppWindow::newWave ( char *filename )
{
	//try to open the file; if it succeeds, fail
	FileAccess *target = new FileAccess ( filename, "r" );
	
	if ( target->errNum() != 0 )
	{
		char msg[BUFSIZE];

		snprintf ( msg, sizeof ( msg ), "%s\nOverwrite denied: file already exists.", filename );
		msg [ sizeof (msg ) - 1 ] = 0;			
		messageDialogManager->post ( msg );
		delete target;
		return;
	}
	
	delete target;

	long numSamples = mixwaveResources->defaultWavNumSamples();
	long samplesPerSec = mixwaveResources->samplesPerSec();
	short bitsPerSample = mixwaveResources->bitsPerSample();
	short channels = mixwaveResources->channels();

	Wave *pWave = new Wave;

	if ( pWave->waveCount() > 1 )
	{
		int count = pWave->waveCount();

		while ( count > 0 )
		{
			if ( pWave->name() != NULL )
			{
				if ( strcmp ( filename, pWave->name() ) == 0 )
				{
					if ( mixwaveResources->enableWarningPopups() == True )
					{
						char msg[BUFSIZE];

						snprintf ( msg, sizeof ( msg ), "Wave %s already exists.", filename );
						msg [ sizeof ( msg ) - 1 ] = 0;						
						messageDialogManager->post ( msg );
					}
					
					pWave = pWave->first();
					delete pWave;
					return;
				}
			}

			if ( count != 1 )
				pWave = pWave->next();

			count--;
		}

		pWave = pWave->first();
	}

	pWave->filePos = 0;

	if ( pWave->AllocateDataSpace ( numSamples * ( bitsPerSample / 8 ) * channels ) == False )
	{
		char msg[BUFSIZE];

		snprintf ( msg, sizeof ( msg ), "%s\nCreate initial %ld zero samples failed\n\
waveclass AllocateDataSpace failed.", pWave->name(), numSamples );
		msg [ sizeof ( msg ) - 1 ] = 0;						
		messageDialogManager->post ( msg );
		delete pWave;
		return;
	}
	else
	{
		// zero data space
		for ( long count=0; count < ( numSamples<<1 * channels ); count++ )
			pWave->data()[count] = 0;
	}

	pWave->FillHeader ( channels, samplesPerSec, bitsPerSample, numSamples );
	pWave->SaveHeader ( filename );

	if ( pWave->waveType() == 1 )
		pWave->filePos = pWave->HeaderSize();	// length of linear wav header
	else
	{
		char msg[BUFSIZE];

		snprintf ( msg, sizeof ( msg ), "%s\nWave type %d unsupported.", pWave->name(), pWave->waveType() );
		msg [ sizeof ( msg ) - 1 ] = 0;
		messageDialogManager->post ( msg );

		delete pWave;
		return;
	}

	pWave->AppendBlock ( filename, channels, samplesPerSec, bitsPerSample, numSamples );

	delete pWave;
	loadWave ( filename );	
}


void AppWindow::loadWaveCallback ( void *clientData, char *filename )
{
	if ( filename[strlen(filename) - 1] != '/' )	// no file specified
	{
		AppWindow *obj = ( AppWindow * ) clientData;
		obj->loadWave ( filename );
	}
}


int AppWindow::loadWave ( char *filename )
{
	char *name = filename;
	char buf[BUFSIZE];
	int error = 0;
	int count = 0;
	long fileSize = 0;
	long numSamples = 0;
	unsigned int fileStatus;

	// open r/w and store state
	FileAccess *wavFile = new FileAccess ( filename, "r+" );
	error = wavFile->errNum();
	fileSize = wavFile->fileSize();
	delete wavFile;

	if ( !error )
		fileStatus = 0;	// read/write
	else
	{
		FileAccess *wavFile = new FileAccess ( filename, "r" );	// at a minimum
		error = wavFile->errNum();
		fileSize = wavFile->fileSize();
		delete wavFile;

		if ( error )
		{
			snprintf ( buf, sizeof ( buf ), "%s\nError: %s", filename, wavFile->error() );
			buf [ sizeof ( buf ) - 1 ] = 0;
			messageDialogManager->post ( buf );
		}
		else
			fileStatus = 1;	// read-only
	}

	if ( error )
		return ( error );

	Wave *pWave = new Wave;	// wave deleted in 'ExitCmd'

	if ( pWave->waveCount() > 1 )
	{
		count = pWave->waveCount();

		while ( count > 0 )
		{
			if ( pWave->name() != NULL )
			{
				if ( strcmp ( name, pWave->name() ) == 0 )
				{
					if ( mixwaveResources->enableWarningPopups() == True )
					{
						snprintf ( buf, sizeof ( buf ), "Wave %s is already loaded.", filename );
						buf [ sizeof ( buf ) - 1 ] = 0;
						messageDialogManager->post ( buf );
					}

					pWave = pWave->first();
					delete pWave;
					return ( -1 );
				}
			}

			if ( count != 1 )
				pWave = pWave->next();

			count--;
		}

		pWave = pWave->first();
	}

	if ( !pWave->LoadHeader ( filename ) )
	{
   		strncpy ( buf, pWave->name(), sizeof ( buf ) );
		strncat ( buf, "\nWave::LoadHeader failed.", sizeof ( buf ) - strlen ( buf ) - 1 );
		messageDialogManager->post ( buf );
		delete pWave;
		return ( 1 );
	}

	if ( strncmp ( "RIFF", ( char * ) pWave->riffHeader(), 4 ) != 0 )
	{
   		strncpy ( buf, pWave->name(), sizeof ( buf ) );
		strncat ( buf, "\nDoesn't appear to be a wave file.", sizeof ( buf ) - strlen ( buf ) - 1 );
		messageDialogManager->post ( buf );
		delete pWave;
		return ( 1 );	
	}

	// crude get-out for nonstandard headers as defined by waveclass
	if ( fileSize != pWave->HeaderSize() + pWave->dataLength() )
	{
  		strncpy ( buf, pWave->name(), sizeof ( buf ) );
		strncat ( buf, "\nUnrecognised wave header.\n", sizeof ( buf ) - strlen ( buf ) - 1 );
		strncat ( buf, "Non-standard header size (>44 bytes).\n", sizeof ( buf ) - strlen ( buf ) - 1 );

		if ( strncmp ( "data", ( char * ) pWave->dataChunkId(), 4 ) != 0 )
		{
			strncat ( buf, "Expected format chunk ID 'data', found '", sizeof ( buf ) - strlen ( buf ) - 1 );
			strncat ( buf, pWave->dataChunkId(), sizeof ( buf ) - strlen ( buf ) - 1 );
			strncat ( buf, "'.\n", sizeof ( buf ) - strlen ( buf ) - 1 );	// pedant...
		}	

		strncat ( buf, "Use 'sox input.wav output.wav' to convert.\n", sizeof ( buf ) - strlen ( buf ) - 1 );

		messageDialogManager->post ( buf );
		delete pWave;
		return ( 1 );	
	}

	// should really be using libaudiofile...
	numSamples = mixwaveResources->blockLength()>>1;
	WavePak temp;

	temp.pWave = pWave;
	temp.pIntArray = NULL;	// no data array for wave
 	temp.graphCmd = NULL;	// no view obj for wave
	temp.graphNum = 0;		// no id unless editor pops up wave view
	temp.startRange = 0;	// no range or selection unless wave view activated
	temp.endRange = 0;
	temp.startSel = 0;
	temp.endSel = 0;
	temp.xCurrent = 0;
	temp.yCurrent = 0;
	temp.dataOffset = 0;	// last data load array[0] w/ref to file pos

	if ( !mixwaveResources->blockLength() )
		temp.intArraySize = pWave->dataLength()>>1;
	else
		temp.intArraySize = numSamples;

	temp.waveDataCmd = NULL;// no wave data view obj
	temp.infoNum = 0; 		// no info panel displayed for this wave yet
	temp.fLock = INACTIVE;
	temp.revision = 0;		// 'no revision'
	temp.waveChanged = False;
	temp.templateNum = 0;	// no template associated with wave
	temp.filePerm = fileStatus;
	temp.gainTemplate = 0;	// no gain templatePak associated with wave
	temp.panTemplate = 0;	// no pan templatePak associated with wave
	temp.gain = mixwaveResources->gain();			// default unity gain
	temp.pan = mixwaveResources->pan();			// default centre pan
	temp.gainRatio = 0;		// ratio of current wave data block to current
	temp.panRatio = 0;		// template data block (dynamic alloc. in MixCmd)
	temp.opMode = 0;		// use all/part wave for mix ( default all )
	wavePak->Add ( temp );

	MixPak	mixtemp;

	mixtemp.waveNum = pWave->waveNum();
	mixtemp.start = 0;
	mixtemp.end = 0;

	mixPak->Add ( mixtemp );

	return ( 0 );
}	


void AppWindow::loadTemplateCallback ( void *clientData, char *filename, char *syncFile )
{
	if ( filename[strlen(filename) - 1] != '/' )	// no file specified
	{
		AppWindow *obj = ( AppWindow * ) clientData;
		obj->loadTemplate ( filename, syncFile );
	}
}


void AppWindow::loadTemplate ( char *filename, char *syncFile )
{
	char text[5];
	int textPtr = 4;
	int srcPtr = strlen ( filename );
	int dataType;
	long numTemplateData;

	for ( int count = 0; count < templatePak->Count(); count++ )
	{
		if ( strcmp ( filename, (*templatePak)[count].name ) == 0 )
		{
			if ( mixwaveResources->enableWarningPopups() == True )
			{
				char buf[BUFSIZE];

				snprintf ( buf, sizeof ( buf ), "template %s is already loaded.", filename );
				buf [ sizeof ( buf ) - 1 ] = 0;
				messageDialogManager->post ( buf );
				return;
			}		
		}
	}

	while ( textPtr > -1 )
	{
		text[textPtr] = filename[srcPtr];
		textPtr--;
		srcPtr--;
	}

	// allocate type on '.int' or '.flt'
	if ( strncmp ( text, ".int", 4 ) == 0 )
		dataType = 0;
	else
		if ( strncmp ( text, ".flt", 4 ) == 0 )
			dataType = 1;
		else
		{
			if ( mixwaveResources->enableMessagePopups() == True )
			{
				char buf[BUFSIZE];

				snprintf ( buf, sizeof ( buf ), "%s\nUnrecognised template type suffix %s\n \
Valid suffixes: '.flt', '.int'.", filename, text );
				buf [ sizeof ( buf ) - 1 ] = 0;
				messageDialogManager->post ( buf );
				return;
			}
		}

	FileAccess *templateFile = new FileAccess ( filename, "rw" );

	if ( templateFile->errNum() != 0 )
	{
		char msg[BUFSIZE<<1];

		snprintf ( msg, sizeof ( msg ), "%s\nError: %s", filename, templateFile->error() );
		msg [ sizeof ( msg ) - 1 ] = 0;
		delete templateFile;
		messageDialogManager->post ( msg );
		return;
	}
	else
	{
		if ( dataType == 0 )
			numTemplateData = templateFile->fileSize() / sizeof ( int );	// int data
		else
			if ( dataType == 1 )
				numTemplateData = templateFile->fileSize() / sizeof ( float );	// float data
	}

	delete templateFile;

	if ( dataType == -1 )	// unrecognised
	{
		char msg[BUFSIZE];

		snprintf( msg, sizeof ( msg ), "%s\nError: data type unrecognised.\nExpecting\n'.int' or '.flt' suffix.",
					filename );
		msg [ sizeof ( msg ) - 1 ] = 0;
		messageDialogManager->post ( msg );
		return;	// no way of identifying what's in the file
	}

	TemplatePak temp;

	temp.templateNum = templatePak->Count() + 1;
	temp.dataType = dataType;
	temp.dataSize = numTemplateData;	// num int or float data elements
	strncpy ( temp.name, filename, sizeof ( temp.name ) );
	temp.name [ sizeof ( temp.name ) - 1 ] = 0;
	temp.syncFile[0] = 0;
	temp.syncRatio = 1;
	temp.graphFltCmd = NULL;
	temp.graphIntCmd = NULL;
	temp.graphNum = 0;
	temp.filePos = 0;		// template equiv. of pWave->filePos
	temp.intArraySize = ( int ) mixwaveResources->blockLength() / sizeof ( int );

	if ( temp.intArraySize == 0 )
		temp.intArraySize = sizeof ( int );

	temp.fltArraySize = ( int ) mixwaveResources->blockLength() / sizeof ( float );

	if ( temp.fltArraySize == 0 )
		temp.fltArraySize = sizeof ( float );

	temp.startRange = 0;	// no range or selection unless view activated
	temp.endRange = 0;
	temp.startSel = 0;
	temp.endSel = 0;
	temp.xCurrent = 0;
	temp.yIntCurrent = 0;
	temp.yFltCurrent = 0.0;
	temp.dataOffset = 0;
	temp.pIntArray = NULL;
	temp.pFloatArray = NULL;
	temp.infoNum = 0;

	if ( dataType == 0 )
	{
		temp.min = ( float ) mixwaveResources->minY();
		temp.max = ( float ) mixwaveResources->maxY();
	}
	else
	{
		temp.min = mixwaveResources->minFltY();
		temp.max = mixwaveResources->maxFltY();
	}

	temp.templateChanged = False;
	temp.fLock = INACTIVE;
	temp.file = NULL;

	templatePak->Add ( temp );

	if ( mixwaveResources->syncTemplateLength() == True )
		if ( syncFile[strlen(syncFile) - 1] != '/' )	// no file specified
			if ( strlen ( syncFile ) != 0 )
				syncSelected ( syncFile );
}


void AppWindow::syncSelected ( char *filename )
{
	int count;
	int found = 0;

	if ( wavePak->Count() > 0 )
	{
		for ( count = 0; count < wavePak->Count(); count++ )
		{
			if ( strcmp ( filename, (*wavePak)[count].pWave->name() ) == 0 )
			{
				found = 1;
				count = wavePak->Count();
			}
		}
	}

	if ( !found )
	{
		int error = loadWave ( filename );

		if ( error )
		{
			char msg[BUFSIZE];

			(*templatePak)[templatePak->Count()-1].syncRatio = 1;
			snprintf( msg, sizeof ( msg ), "%s\nError: not a wave file: can't sync. length to \
template.\nTime-synchronised view disabled.", filename );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	// there is a wave for the template, locate its pak
	for ( int count = 0; count < wavePak->Count(); count++ )
	{
		if ( strcmp ( filename, (*wavePak)[count].pWave->name() ) == 0 )
		{
			(*wavePak)[count].templateNum = (*templatePak)[templatePak->Count()-1].templateNum;

			// sync. ratio is num wave samples represented by each template datum
			if ( (*wavePak)[count].pWave->totalSamples() > 0 )
			{
				(*templatePak)[templatePak->Count()-1].syncRatio = ( float ) (*wavePak)[count].pWave->totalSamples() / ( float ) (*templatePak)[templatePak->Count()-1].dataSize;
				strncpy ( (*templatePak)[templatePak->Count()-1].syncFile, filename, sizeof ( (*templatePak)[templatePak->Count()-1].syncFile ) );
				(*templatePak)[templatePak->Count()-1].syncFile [ sizeof ( (*templatePak)[templatePak->Count()-1].syncFile ) - 1 ] = 0;
			}
			else
				(*templatePak)[templatePak->Count()-1].syncFile[0] = 0;
		}
	}
}


void AppWindow::fileSelectHelpCallback ( XtPointer callData )
{
	XmFileSelectionBoxCallbackStruct *cb = ( XmFileSelectionBoxCallbackStruct * ) callData;
	char *name = NULL;
	XmString xmstr = cb->dir;	// selected file
	int status = 0;

	if ( xmstr )	// ensure file was selected
	{
		// extract first char. string matching default character set from
		// compound string
		status = XmStringGetLtoR ( xmstr, XmFONTLIST_DEFAULT_TAG, &name );

//		if ( status )	// successfully extracted string
//			obj->selectedFile ( name );

		XmStringFree ( xmstr );
	}
}


void AppWindow::fileSelectCancelledCallback ( XtPointer callData )
{
	// unused
}


void AppWindow::textCallback ( XtPointer callData )
{
	// unused
}


void AppWindow::textCancelledCallback ( XtPointer callData )
{
	// unused
}


void AppWindow::mixdownCallback ( void *clientData, char *filename )
{
	if ( filename[strlen(filename) - 1] != '/' )	// no file specified
	{
		AppWindow *obj = ( AppWindow * ) clientData;

		if ( mixwaveResources->mixing() == True )
		{
			 if ( mixwaveResources->enableWarningPopups() == True )
			 {
				char msg[BUFSIZE];

				snprintf ( msg, sizeof ( msg ), "%s\nMixdown already in progress!", filename );
				msg [ sizeof (msg ) - 1 ] = 0;			
				messageDialogManager->post ( msg );
			 }
		}
		else
			obj->mixdown ( filename );
	}
}


void AppWindow::mixdown ( char *mixName )
{
	FileAccess *target = new FileAccess ( mixName, "r" );
	
	if ( target->errNum() == 0 )
	{
		char msg[BUFSIZE];

		snprintf ( msg, sizeof ( msg ), "%s\nOverwrite denied: delete or rename mix.", mixName );
		msg [ sizeof (msg ) - 1 ] = 0;			
		messageDialogManager->post ( msg );
	}
	else
	{
		mixwaveResources->mixing ( True );
		MixCmd *mixCmd = new MixCmd ( "Mix", TRUE, mixName );
		setBusyCursor();
		mixCmd->execute ( &AppWindow::mixFinishedCallback, ( void * ) this );
	}

	delete target;
}


void AppWindow::mixFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	MixCmd *mixObj = ( MixCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableMessagePopups() == True )
			messageDialogManager->post ( "Mixdown interrupted!" );
	}
	else		
		obj->mixFinished ( mixObj );

	obj->setNormalCursor();
	delete mixObj;
	mixwaveResources->mixing ( False );

	// set file locks inactive ( left locked if work procedure interrupted )
	int n = 0;

	while ( n < wavePak->Count() )
	{
		(*wavePak)[n].fLock = INACTIVE; n++;
	}

	n = 0;

	while ( n < templatePak->Count() )
	{
		(*templatePak)[n].fLock = INACTIVE; n++;
	}
}


void AppWindow::mixFinished ( MixCmd *mixObj )
{
 	char msg[BUFSIZE];

	msg[0] = 0;

	if ( strlen ( mixObj->error() ) != 0 )
	{
		snprintf ( msg, sizeof ( msg ), "%s", mixObj->error() );
		msg [ sizeof ( msg ) - 1 ] = 0;		
		messageDialogManager->post ( msg );
	}
	else
	{
		 if ( mixwaveResources->enableMessagePopups() == True )
		 {
			char intText [ 20 ];
			snprintf ( msg, BUFSIZE, "%s\nMixdown completed.", mixObj->mixName() );
			msg [ sizeof ( msg ) - 1 ] = 0;						
			strncat ( msg, " Max. bits used ", sizeof ( msg ) - strlen ( msg ) - 1 );
			msg [ sizeof ( msg ) - 1 ] = 0;
			snprintf ( intText, sizeof ( intText ), "%lu", mixObj->maxMixBits() );
			intText [ sizeof ( intText ) - 1 ] = 0;
			strncat ( msg, intText, sizeof ( msg ) - strlen ( msg ) - 1 );
			msg [ sizeof ( msg ) - 1 ] = 0;
			strncat ( msg, ".", sizeof ( msg ) - strlen ( msg ) - 1 );
			msg [ sizeof ( msg ) - 1 ] = 0;

			if ( mixObj->maxMixBits() > 16 )
			{
				strncat ( msg, "\nTruncated ", sizeof ( msg ) - strlen ( msg ) - 1 );
				msg [ sizeof ( msg ) - 1 ] = 0;
				snprintf ( intText, sizeof ( intText ), "%lu", mixObj->lostBits() );
				intText [ sizeof ( intText ) - 1 ] = 0;
				strncat ( msg, intText, sizeof ( msg ) - strlen ( msg ) - 1 );
				msg [ sizeof ( msg ) - 1 ] = 0;

				if ( mixObj->lostBits() > 1 )
					strncat ( msg, " bits.", sizeof ( msg ) - strlen ( msg ) - 1 );
				else
					strncat ( msg, " bit.", sizeof ( msg ) - strlen ( msg ) - 1 );

				msg [ sizeof ( msg ) - 1 ] = 0;
			}

			infoDialogManager->post ( msg );
		}
	}
}


void AppWindow::newSpatialTemplateCallback ( void *clientData, char *baseFileName, unsigned int callbackReason )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->newSpatialTemplate ( baseFileName, callbackReason );
}


void AppWindow:: newSpatialTemplate ( char *baseFileName, unsigned int callbackReason )
{
	if ( callbackReason == 3 ) // process error, 1 = 'Ok', 2 = 'Cancel'
	{
		MixwaveUtils *utils = new MixwaveUtils;
		char command [ 2 * ( strlen ( baseFileName ) + strlen (
		mixwaveResources->removeCmd() ) + 18 ) ];
		strncpy ( command, mixwaveResources->removeCmd(), sizeof ( command ) );

		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, " ", sizeof ( command ) - strlen ( command ) - 1 ) ;
		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, baseFileName, sizeof ( command ) - strlen ( command ) - 1 );
		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, "*.flt; ", sizeof ( command ) - strlen ( command ) - 1 );
		command [ sizeof ( command ) - 1 ] = 0;

		strncat ( command, mixwaveResources->removeCmd(), sizeof ( command ) - strlen ( command ) - 1 );
		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, " ", sizeof ( command ) - strlen ( command ) - 1 ) ;
		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, baseFileName, sizeof ( command ) - strlen ( command ) - 1 );
		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, "_pakData", sizeof ( command ) - strlen ( command ) - 1 );
		command [ sizeof ( command ) - 1 ] = 0;

		utils->sysCall ( command );
		delete utils;
	}
}


void AppWindow::asciiToFloatCallback ( void *clientData, char *txtFile, char *floatFile )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->asciiToFloat ( txtFile, floatFile ); 
}


void AppWindow::asciiToHetroCallback ( void *clientData, char *txtFile, char *hetFile )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->asciiToHetro ( txtFile, hetFile ); 
}


void AppWindow::wavToRawCallback ( void *clientData, char *wavFile, char *rawFile )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->wavToRaw ( wavFile, rawFile ); 
}


void AppWindow::rawToWavCallback ( void *clientData, char *rawFile, char *wavFile )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->rawToWav ( rawFile, wavFile ); 
}


void AppWindow::intToWavCallback ( void *clientData, char *intFile, char *wavFile )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->intToWav ( intFile, wavFile ); 
}


void AppWindow::wavToIntCallback ( void *clientData, char *wavFile, char *intFile )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->wavToInt ( wavFile, intFile ); 
}


void AppWindow::intToFloatCallback ( void *clientData, char *intFile, char *floatFile )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->intToFloat ( intFile, floatFile ); 
}


void AppWindow::floatToAsciiCallback ( void *clientData, char *floatFile, char *txtFile )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->floatToAscii ( floatFile, txtFile ); 
}


void AppWindow::hetroToAsciiCallback ( void *clientData, char *hetFile, char *txtFile )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->hetroToAscii ( hetFile, txtFile ); 
}


void AppWindow::floatToIntCallback ( void *clientData, char *floatFile, char *intFile )
{
	AppWindow *obj = ( AppWindow * ) clientData;

	obj->floatToInt ( floatFile, intFile ); 
}


void AppWindow::wavToRaw ( char *wavFile, char *rawFile )
{
	if ( strlen ( rawFile ) == 0 )
		return;

	// is the wavfile of this name active? - check v list
	if ( wavePak->Count() > 0 )
	{
		for ( int count = 0; count < wavePak->Count(); count++ )
		{
			if ( strcmp ( (*wavePak)[count].pWave->name(), wavFile ) == 0 )
			{
				if ( (*wavePak)[count].fLock == ACTIVE )
					return;
				else
					(*wavePak)[count].fLock = ACTIVE;
			}
		}
	}

	WavToRawCmd *wtrCmd = new WavToRawCmd ( "WavToRaw", TRUE, wavFile, rawFile );

	setBusyCursor();
	wtrCmd->execute ( &AppWindow::wavToRawFinishedCallback, ( void * ) this );
}


void AppWindow::wavToRawFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	WavToRawCmd *wtrObj = ( WavToRawCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Wave to raw conversion interrupted!" );
	}
	else
		obj->wavToRawFinished ( wtrObj );

	obj->setNormalCursor();
	delete wtrObj;
}


void AppWindow::wavToRawFinished ( WavToRawCmd *wtrObj )
{
	 if ( mixwaveResources->enableMessagePopups() == True )
	 {
		if ( wtrObj->error() == 0 )
		{
		 	char msg[BUFSIZE<<1];

			snprintf ( msg, sizeof ( msg ), "%s\nWave converted to raw:\n%s", wtrObj->source(), wtrObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}

	// is the wavfile of this name loaded? - check v list and set INACTIVE
	if ( wavePak->Count() > 0 )
	{
		for ( int count = 0; count < wavePak->Count(); count++ )
		{
			if ( strcmp ( (*wavePak)[count].pWave->name(), wtrObj->source() ) == 0 )
			{
				(*wavePak)[count].fLock = INACTIVE;
				count = wavePak->Count();
			}
		}
	}
}


void AppWindow::rawToWav ( char *rawFile, char *wavFile )
{
	if ( strlen ( wavFile ) == 0 )
		return;

	RawToWavCmd *rtwCmd = new RawToWavCmd ( "rawToWav", TRUE, rawFile, wavFile );

	rtwCmd->setWaveDefaults ( mixwaveResources->samplesPerSec(),
								mixwaveResources->bitsPerSample(),
								mixwaveResources->channels() );

	setBusyCursor();
	rtwCmd->execute ( &AppWindow::rawToWavFinishedCallback, ( void * ) this );
}


void AppWindow::rawToWavFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	RawToWavCmd *rtwObj = ( RawToWavCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Raw to wave conversion interrupted!" );
	}
	else
		obj->rawToWavFinished ( rtwObj );

	obj->setNormalCursor();
	delete rtwObj;
}


void AppWindow::rawToWavFinished ( RawToWavCmd *rtwObj )
{
	 if ( mixwaveResources->enableMessagePopups() == True )
	 {
		if ( rtwObj->error() == 0 )
		{
		 	char msg[2048];

			snprintf ( msg, sizeof ( msg ), "%s\nRaw converted to wave:\n%s", rtwObj->source(), rtwObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}
}


void AppWindow::wavToInt ( char *wavFile, char *intFile )
{
	if ( strlen ( intFile ) == 0 )
		return;

	// is the wavfile of this name active? - check v list
	if ( wavePak->Count() > 0 )
	{
		for ( int count = 0; count < wavePak->Count(); count++ )
		{
			if ( strcmp ( (*wavePak)[count].pWave->name(), wavFile ) == 0 )
			{
				if ( (*wavePak)[count].fLock == ACTIVE )
					return;
				else
					(*wavePak)[count].fLock = ACTIVE;
			}
		}
	}

	WavToIntCmd *wtiCmd = new WavToIntCmd ( "WavToInt", TRUE, wavFile, intFile );

	wtiCmd->setWaveDefaults ( mixwaveResources->samplesPerSec(),
								mixwaveResources->bitsPerSample(),
								mixwaveResources->channels() );

	wtiCmd->offset ( mixwaveResources->convertOffsetInt() );
	wtiCmd->scale ( mixwaveResources->convertScaleInt() );
	setBusyCursor();
	wtiCmd->execute ( &AppWindow::wavToIntFinishedCallback, ( void * ) this );
}


void AppWindow::wavToIntFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	WavToIntCmd *wtiObj = ( WavToIntCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Wave to integer conversion interrupted!" );
	}
	else
		obj->wavToIntFinished ( wtiObj );

	obj->setNormalCursor();
	delete wtiObj;
}


void AppWindow::wavToIntFinished ( WavToIntCmd *wtiObj )
{
	 if ( mixwaveResources->enableMessagePopups() == True )
	 {
		if ( wtiObj->error() == 0 )
		{
		 	char msg[2048];

			snprintf ( msg, sizeof ( msg ), "%s\nWave converted to integer:\n%s", wtiObj->source(), wtiObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}

	if ( wavePak->Count() > 0 )
	{
		for ( int count = 0; count < wavePak->Count(); count++ )
		{
			if ( strcmp ( (*wavePak)[count].pWave->name(), wtiObj->source() ) == 0 )
			{
				(*wavePak)[count].fLock = INACTIVE;
				count = wavePak->Count();
			}
		}
	}
}


void AppWindow::intToWav ( char *intFile, char *wavFile )
{
	if ( strlen ( intFile ) == 0 )
		return;

	// is the wavfile of this name active? - check v list
	if ( wavePak->Count() > 0 )
	{
		for ( int count = 0; count < wavePak->Count(); count++ )
		{
			if ( strcmp ( (*wavePak)[count].pWave->name(), wavFile ) == 0 )
			{
				if ( (*wavePak)[count].fLock == ACTIVE )
					return;
				else
					(*wavePak)[count].fLock = ACTIVE;
			}
		}
	}

	IntToWavCmd *itwCmd = new IntToWavCmd ( "intToWav", TRUE, intFile, wavFile );

	itwCmd->setWaveDefaults ( mixwaveResources->samplesPerSec(),
								mixwaveResources->bitsPerSample(),
								mixwaveResources->channels() );
	setBusyCursor();
	itwCmd->execute ( &AppWindow::intToWavFinishedCallback, ( void * ) this );
}


void AppWindow::intToWavFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	IntToWavCmd *itwobj = ( IntToWavCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Integer to wave conversion interrupted!" );
	}
	else
		obj->intToWavFinished ( itwobj );

	obj->setNormalCursor();
	delete itwobj;
}


void AppWindow::intToWavFinished ( IntToWavCmd *itwobj )
{
	 if ( mixwaveResources->enableMessagePopups() == True )
	 {
		if ( itwobj->error() == 0 )
		{
		 	char msg[2048];

			snprintf ( msg, sizeof ( msg ), "%s\nInteger converted to wave:\n%s", itwobj->source(), itwobj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}

	if ( wavePak->Count() > 0 )
	{
		for ( int count = 0; count < wavePak->Count(); count++ )
		{
			if ( strcmp ( (*wavePak)[count].pWave->name(), itwobj->source() ) == 0 )
			{
				(*wavePak)[count].fLock = INACTIVE;
				count = wavePak->Count();
			}
		}
	}
}


void AppWindow::asciiToFloat ( char *txtFile, char *floatFile )
{	
	if ( strlen ( floatFile ) == 0 )
		return;

	AsciiToFloatCmd *atfCmd = new AsciiToFloatCmd ( "asciiToFloat", TRUE, txtFile, floatFile );

	atfCmd->offset ( mixwaveResources->convertOffsetFloat() );
	atfCmd->scale ( mixwaveResources->convertScaleFloat() );
	setBusyCursor();
	atfCmd->execute ( &AppWindow::asciiToFloatFinishedCallback, ( void * ) this );
}


void AppWindow::asciiToFloatFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	AsciiToFloatCmd *atfObj = ( AsciiToFloatCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Ascii to floating point conversion interrupted!" );
	}
	else
		obj->asciiToFloatFinished ( atfObj );

	obj->setNormalCursor();
	delete atfObj;
}


void AppWindow::asciiToFloatFinished ( AsciiToFloatCmd *atfObj )
{
	if ( mixwaveResources->enableMessagePopups() == True )
	{
		if ( atfObj->error() == 0 )
		{
			char msg[BUFSIZE<<1];

			snprintf ( msg, sizeof ( msg ), "%s\nAscii converted to floating point:\n%s", atfObj->source(), atfObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}
}


void AppWindow::asciiToHetro ( char *txtFile, char *hetFile )
{	
	if ( strlen ( hetFile ) == 0 )
		return;

	AsciiToHetroCmd *athCmd = new AsciiToHetroCmd ( "asciiToHetro", TRUE, txtFile, hetFile );

	setBusyCursor();
	athCmd->execute ( &AppWindow::asciiToHetroFinishedCallback, ( void * ) this );
}


void AppWindow::asciiToHetroFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	AsciiToHetroCmd *athObj = ( AsciiToHetroCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Ascii to hetro conversion interrupted!" );
	}
	else
		obj->asciiToHetroFinished ( athObj );

	obj->setNormalCursor();
	delete athObj;
}


void AppWindow::asciiToHetroFinished ( AsciiToHetroCmd *athObj )
{
	if ( mixwaveResources->enableMessagePopups() == True )
	{
		if ( athObj->error() == 0 )
		{
			char msg[BUFSIZE<<1];

			snprintf ( msg, sizeof ( msg ), "%s\nAscii converted to hetro:\n%s", athObj->source(), athObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}
}


void AppWindow::intToFloat ( char *intFile, char *floatFile )
{	
	if ( strlen ( floatFile ) == 0 )
		return;

	IntToFloatCmd *itfCmd = new IntToFloatCmd ( "intToFloat", TRUE, intFile, floatFile );

	itfCmd->offset ( mixwaveResources->convertOffsetFloat() );
	itfCmd->scale ( mixwaveResources->convertScaleFloat() );
	setBusyCursor();
	itfCmd->execute ( &AppWindow::intToFloatFinishedCallback, ( void * ) this );
}


void AppWindow::intToFloatFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	IntToFloatCmd *itfObj = ( IntToFloatCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Integer to floating point conversion interrupted!" );
	}
	else
		obj->intToFloatFinished ( itfObj );

	obj->setNormalCursor();
	delete itfObj;
}


void AppWindow::intToFloatFinished ( IntToFloatCmd *itfObj )
{
	 if ( mixwaveResources->enableMessagePopups() == True )
	 {
		if ( itfObj->error() == 0 )
		{
		 	char msg[BUFSIZE<<1];

			snprintf ( msg, sizeof ( msg ), "%s\nInteger converted to floating point:\n%s", itfObj->source(), itfObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}
}


void AppWindow::floatToAscii ( char *floatFile, char *txtFile )
{	
	if ( strlen ( txtFile ) == 0 )
		return;

	FloatToAsciiCmd *ftaCmd = new FloatToAsciiCmd ( "FloatToAscii", TRUE, floatFile, txtFile );

	ftaCmd->offset ( mixwaveResources->convertOffsetInt() );
	ftaCmd->scale ( mixwaveResources->convertScaleInt() );
	ftaCmd->fpDigits ( mixwaveResources->fpDigits() );
	setBusyCursor();
	ftaCmd->execute ( &AppWindow::floatToAsciiFinishedCallback, ( void * ) this );
}


void AppWindow::floatToAsciiFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	FloatToAsciiCmd *ftaObj = ( FloatToAsciiCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Floating point to ascii conversion interrupted!" );
	}
	else
		obj->floatToAsciiFinished ( ftaObj );

	obj->setNormalCursor();
	delete ftaObj;
}


void AppWindow::floatToAsciiFinished ( FloatToAsciiCmd *ftaObj )
{
	 if ( mixwaveResources->enableMessagePopups() == True )
	 {
		if ( ftaObj->error() == 0 )
		{
		 	char msg[BUFSIZE<<1];

			snprintf ( msg, sizeof ( msg ), "%s\nFloating point converted to ascii:\n%s", ftaObj->source(), ftaObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}
}


void AppWindow::hetroToAscii ( char *hetFile, char *txtFile )
{	
	if ( strlen ( txtFile ) == 0 )
		return;

	HetroToAsciiCmd *htaCmd = new HetroToAsciiCmd ( "hetroToAscii", TRUE, hetFile, txtFile );

	setBusyCursor();
	htaCmd->execute ( &AppWindow::hetroToAsciiFinishedCallback, ( void * ) this );
}


void AppWindow::hetroToAsciiFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	HetroToAsciiCmd *htaObj = ( HetroToAsciiCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Hetro to ascii conversion interrupted!" );
	}
	else
		obj->hetroToAsciiFinished ( htaObj );

	obj->setNormalCursor();
	delete htaObj;
}


void AppWindow::hetroToAsciiFinished ( HetroToAsciiCmd *htaObj )
{
	 if ( mixwaveResources->enableMessagePopups() == True )
	 {
		if ( htaObj->error() == 0 )
		{
		 	char msg[BUFSIZE<<1];

			snprintf ( msg, sizeof ( msg ), "%s\nHetro converted to ascii:\n%s", htaObj->source(), htaObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}
}


void AppWindow::floatToInt ( char *floatFile, char *intFile )
{	
	if ( strlen ( intFile ) == 0 )
		return;

	FloatToIntCmd *ftiCmd = new FloatToIntCmd ( "FloatToInt", TRUE, floatFile, intFile );

	ftiCmd->offset ( mixwaveResources->convertOffsetInt() );
	ftiCmd->scale ( mixwaveResources->convertScaleInt() );
	setBusyCursor();
	ftiCmd->execute ( &AppWindow::floatToIntFinishedCallback, ( void * ) this );
}


void AppWindow::floatToIntFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	FloatToIntCmd *ftiObj = ( FloatToIntCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Floating point to integer conversion interrupted!" );
	}
	else
		obj->floatToIntFinished ( ftiObj );

	obj->setNormalCursor();
	delete ftiObj;
}


void AppWindow::floatToIntFinished ( FloatToIntCmd *ftiObj )
{
	 if ( mixwaveResources->enableMessagePopups() == True )
	 {
		if ( ftiObj->error() == 0 )
		{
		 	char msg[BUFSIZE<<1];

			snprintf ( msg, sizeof ( msg ), "%s\nFloating point converted to integer:\n%s", ftiObj->source(), ftiObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}
}


void AppWindow::setBusyCursor()
{
	if ( XtIsRealized ( _w ) )
	{
		if ( !_busyCursor )
			_busyCursor = XCreateFontCursor ( XtDisplay ( _w ), XC_watch );

		// install cursor for window's top-level shell
		XDefineCursor ( XtDisplay ( _w ), XtWindow ( _w ),  _busyCursor );
	}
}


void AppWindow::setNormalCursor()
{
	if ( XtIsRealized ( _w ) )
	{
		if ( !_normalCursor )
			_normalCursor = XCreateFontCursor ( XtDisplay ( _w ), XC_left_ptr );

		// install left ptr cursor as normal cursor for window's top-level shell
		XDefineCursor ( XtDisplay ( _w ), XtWindow ( _w ), _normalCursor );
	}
}

// n.b. we can't use busy cursor with splitWave since we don't have access
// to the AppWindow object originating the SplitWaveCmd instance inside
// the infoDialogManager callback indicating 'files already exist'.
// file check should be implemented inside SplitWaveCmd... then tidy up here!

void AppWindow::splitWaveCallback ( void *clientData, char *filename )
{
	if ( filename[strlen(filename) - 1] != '/' )	// no file specified
	{
		AppWindow *obj = ( AppWindow * ) clientData;
		obj->splitWave ( filename );
	}
}


void AppWindow::splitWave ( char *filename )
{
	// is the wavfile of this name active? - check v list
	if ( wavePak->Count() > 0 )
	{
		for ( int count = 0; count < wavePak->Count(); count++ )
		{
			if ( strcmp ( (*wavePak)[count].pWave->name(), filename ) == 0 )
			{
				if ( (*wavePak)[count].fLock == ACTIVE )
					return;
			}
		}
	}

	SplitWaveCmd *swCmd = new SplitWaveCmd ( "Split Wave", TRUE, filename,
												mixwaveResources->blockLength() );
	int error = 0;

	if ( mixwaveResources->enableWarningPopups() == True )
	{
		FileAccess *targetL = new FileAccess ( swCmd->targetL(), "r" );
		error = targetL->errNum();	
		delete targetL;

		FileAccess *targetR = new FileAccess ( swCmd->targetR(), "r" );
		error += targetR->errNum();	
		delete targetR;

		if ( error == 0 )	// succeeded, file(s) already exist
		{
			char msg[BUFSIZE<<1];

			snprintf ( msg, sizeof ( msg ), "%s\n%s\nFiles exist: overwrite?", swCmd->targetL(), swCmd->targetR() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg, swCmd,
										&AppWindow::splitWaveOkCallback,
										&AppWindow::splitWaveCancelCallback );
		}
		else
			swCmd->execute ( &AppWindow::splitWaveFinishedCallback, ( void * ) this );		
	}
	else
		swCmd->execute ( &AppWindow::splitWaveFinishedCallback, ( void * ) this );
}


void AppWindow::splitWaveOkCallback ( void *clientData )
{
	SplitWaveCmd *swCmd = ( SplitWaveCmd * ) clientData;
	AppWindow *obj = ( AppWindow * ) clientData;	// dummy obj to get at class methods
													// but we lose ability to setNormalCursor()
	obj->splitWaveOk ( swCmd );	
}


void AppWindow::splitWaveOk ( SplitWaveCmd *swCmd )
{
	swCmd->execute ( &AppWindow::splitWaveFinishedCallback, ( void * ) this );
}


void AppWindow::splitWaveCancelCallback ( void *clientData )
{
	SplitWaveCmd *obj = ( SplitWaveCmd * ) clientData;

	delete obj;
}


void AppWindow::splitWaveFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	SplitWaveCmd *swObj = ( SplitWaveCmd * ) cmd;
	AppWindow *obj = ( AppWindow * ) clientData;

	if ( interrupted )
	{
		if ( mixwaveResources->enableWarningPopups() == True )
			messageDialogManager->post ( "Stereo to mono split interrupted!" );
	}
	else
		obj->splitWaveFinished ( swObj );

	delete swObj;
}


void AppWindow::splitWaveFinished ( SplitWaveCmd *swObj )
{
	 if ( mixwaveResources->enableMessagePopups() == True )
	 {
	 	if ( swObj->error() == 0 )
		{
		 	char msg[3*BUFSIZE];

			snprintf ( msg, sizeof ( msg ), "%s\nStereo wave split to mono:\n%s\n%s",
						swObj->source(), swObj->targetL(), swObj->targetR() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}

	if ( wavePak->Count() > 0 )
	{
		for ( int count = 0; count < wavePak->Count(); count++ )
		{
			if ( strcmp ( (*wavePak)[count].pWave->name(), swObj->source() ) == 0 )
			{
				(*wavePak)[count].fLock = INACTIVE;
				count = wavePak->Count();
			}		
		}
	}
}
