/*
	DeleteCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk             
*/

#include <DMenuCmd.h>
#include "DeleteCmd.h"
#include "AppWindow.h"
#include <GraphFltCmd.h>
#include <GraphIntCmd.h>
#include <MessageDialogManager.h>


DeleteCmd::DeleteCmd ( const char *name, int active, int fileType ) : NoUndoCmd ( name, active )
{
	if ( ( fileType < WAVE ) || ( fileType > TEMPLATE ) )
		return;

	_fileType = fileType;		
	_popupRealised = False;
}


void DeleteCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	unsigned int n;
	unsigned int maxNameLength = 0;

	switch ( _fileType )
	{
		case WAVE:	
			if ( wavePak->Count() == 0 )
				return;

	if ( mixwaveResources->waveCtrlRealised() == True )
	{
		char msg[58];
		strcpy ( msg, "Delete rejected - please close wave control menu first." );
		messageDialogManager->post ( msg );
		return;
	}

			_arrayItems = wavePak->Count();

			for ( n = 0; n < _arrayItems; n++ )
				if ( strlen ( (*wavePak)[n].pWave->name() ) > maxNameLength )
					maxNameLength = strlen ( (*wavePak)[n].pWave->name() );

			break;
			
		case TEMPLATE:
			if ( templatePak->Count() == 0 )
				return;

			_arrayItems = templatePak->Count();
	
			for ( n = 0; n < _arrayItems; n++ )
				if ( strlen ( (*templatePak)[n].name ) > maxNameLength )
					maxNameLength = strlen ( (*templatePak)[n].name );
			
			break;
	}

	n = 0;
	maxNameLength++;

	if ( maxNameLength < 5 )
		maxNameLength = 5;	// accomodate 'quit' at a minimum
		
	_arrayItems++;
	
	char strings [ _arrayItems ] [ maxNameLength ];
	char msg[16];

	switch ( _fileType )
	{
		case WAVE:
			for ( n = 0; n < ( _arrayItems - 1 ); n++ )
				strcpy ( ( char * ) &strings [ n ], (*wavePak)[n].pWave->name() );
				
			strcpy ( msg, "Delete wave" );	
			break;

		case TEMPLATE:
			for ( n = 0; n < ( _arrayItems - 1 ); n++ )
				strcpy ( ( char * ) &strings [ n ], (*templatePak)[n].name );

			strcpy ( msg, "Delete template" );
			break;
	}

	strcpy ( strings [ n ], "Quit" );

	_menu = new DMenuCmd ( msg, TRUE, TRUE,
								&DeleteCmd::srcMenuSelectionCallback,
								( void * ) this, "Delete loaded file:",
								_arrayItems, maxNameLength, ( char * ) &strings );

	_popupRealised = True;
	_menu->execute();
}


void DeleteCmd::srcMenuSelectionCallback ( void *clientData, unsigned int selectDeleteNum, char *label )
{
	DeleteCmd *obj = ( DeleteCmd * ) clientData;				

	if ( selectDeleteNum < obj->_arrayItems )	// 'quit' selected
			obj->deleteItem ( selectDeleteNum );
	
	delete obj->_menu;
	obj->_popupRealised = False;
}


void DeleteCmd::deleteItem ( unsigned int selectDeleteNum )
{
	if ( selectDeleteNum == 0 )
		return;

	unsigned int count, count2;

	switch ( _fileType )
	{
		case WAVE:
		{
			unsigned int numWaves = ( unsigned int ) wavePak->Count();
			int error = 0;
			unsigned int deletedTemplateNum = 0;
			unsigned int deletedWaveNum = 0;
			
			for ( count = 0; count < numWaves; count++ )
			{
				if ( (*wavePak)[count].pWave->waveNum() == selectDeleteNum )
				{	// delete wavePak struct with relevant wave
					if ( (*wavePak)[count].fLock == ACTIVE )
						error++;
					else
					{	// check for a dependent template
						if ( ( templatePak->Count() != 0 ) && ( (*wavePak)[count].templateNum > 0 ) )
						{
							if ( (*templatePak)[(*wavePak)[count].templateNum-1].fLock == ACTIVE )
								error++;
							else
							{	// remove template
								if ( (*templatePak)[(*wavePak)[count].templateNum-1].dataType == 0 )
								{
									if ( (*templatePak)[(*wavePak)[count].templateNum-1].graphIntCmd != NULL )
									{
										(*templatePak)[(*wavePak)[count].templateNum-1].graphIntCmd->close();
										delete (*templatePak)[(*wavePak)[count].templateNum-1].graphIntCmd;
									}
								}
								else
								{
									if ( (*templatePak)[(*wavePak)[count].templateNum-1].graphFltCmd != NULL )
									{
										(*templatePak)[(*wavePak)[count].templateNum-1].graphFltCmd->close();
										delete (*templatePak)[(*wavePak)[count].templateNum-1].graphFltCmd;
									}
								}

								if ( (*templatePak)[(*wavePak)[count].templateNum-1].pIntArray != NULL )
									delete (*templatePak)[(*wavePak)[count].templateNum-1].pIntArray;
								
								if ( (*templatePak)[(*wavePak)[count].templateNum-1].pFloatArray != NULL )
									delete (*templatePak)[(*wavePak)[count].templateNum-1].pFloatArray;								

								deletedTemplateNum = (*wavePak)[count].templateNum;
								templatePak->RemoveItemAt ( (*wavePak)[count].templateNum - 1 );
							}
						}
							
						if ( !error )	// or waiting for dependent template == INACTIVE
						{
							if ( (*wavePak)[count].graphCmd != NULL )
							{
								(*wavePak)[count].graphCmd->close();
								delete (*wavePak)[count].graphCmd;
							}

							if ( (*wavePak)[count].pIntArray != NULL )
							{
								delete (*wavePak)[count].pIntArray;
								(*wavePak)[count].pIntArray = NULL;
							}

							if ( (*wavePak)[count].waveDataCmd != NULL )
							{
								(*wavePak)[count].waveDataCmd->cancel();
								delete (*wavePak)[count].waveDataCmd;
							}

							// find and remove related mixPak for this wave if the former exists;
							for ( count2 = 0; count2 < ( unsigned int ) mixPak->Count(); count2++ )
								if ( (*mixPak)[count2].waveNum == (*wavePak)[count].pWave->waveNum() )
									mixPak->RemoveItemAt ( count2 );

							deletedWaveNum = (*wavePak)[count].pWave->waveNum();
							delete (*wavePak)[count].pWave;
							wavePak->RemoveItemAt ( count );
						}
					}

					while ( count < numWaves )
					{
						// change remaining template IDs if they are wave-linked
						if ( ( (*wavePak)[count].templateNum != 0 )	&&
							( (*wavePak)[count].templateNum >= deletedTemplateNum )
								&& ( deletedTemplateNum != 0 ) ) 
						{	// template exists for wave after deleted item in list
							(*wavePak)[count].templateNum--;
							// adjust templatePak template ID

							(*templatePak)[(*wavePak)[count].templateNum-1].templateNum--;

							// adjust graph, info IDs
							if ( (*templatePak)[(*wavePak)[count].templateNum-1].graphNum != 0 )
							{
								(*templatePak)[(*wavePak)[count].templateNum-1].graphNum--;
						
								// reset graph ID in graph object and template pak
								if ( (*templatePak)[(*wavePak)[count].templateNum-1].dataType == 0 )
									(*templatePak)[(*wavePak)[count].templateNum-1].graphIntCmd->graphNum (
										(*templatePak)[(*wavePak)[count].templateNum-1].graphNum );
								else
									(*templatePak)[(*wavePak)[count].templateNum-1].graphFltCmd->graphNum (
										(*templatePak)[(*wavePak)[count].templateNum-1].graphNum );
							}

						}

						if ( (*templatePak)[(*wavePak)[count].templateNum-1].infoNum != 0 )
							(*templatePak)[(*wavePak)[count].templateNum-1].infoNum--;

						count++;
					}
				}			
			}	

			// reallocate wave IDs to be in sequence
			for ( count = 0 ; count < ( unsigned int ) wavePak->Count(); count++ )
			{
				(*wavePak)[count].pWave->waveNum ( count + 1 );	// wave numbers are sequential

				if ( mixPak->Count() > 0 )	// i.e. has been created by MixCmd
					(*mixPak)[count].waveNum = count + 1;			// mix wave numbers correspond

				if ( ( ( count + 1 ) >= deletedWaveNum ) && ( (*wavePak)[count].infoNum != 0 ) )
					(*wavePak)[count].infoNum--;

				if ( (*wavePak)[count].graphCmd != NULL )
				{
					(*wavePak)[count].graphNum = count + 1;	// reset in wavePak
					(*wavePak)[count].graphCmd->graphNum ( count + 1 );	// re-sync graph/wave IDs
				}

				if ( (*wavePak)[count].waveDataCmd != NULL )
					(*wavePak)[count].waveDataCmd->waveDataNum ( count + 1 );
			}
		}
		
		break;

		case TEMPLATE:
		{
			int error = 0;

			for ( count = 0; count < ( unsigned int ) templatePak->Count(); count++ )
			{
				if ( strcmp ( (*templatePak)[count].name, (*templatePak)[selectDeleteNum-1].name ) == 0 )
				{	// delete templatePak struct
					if ( (*templatePak)[count].fLock == ACTIVE )
						error++;
					else
					{	
						// check for linked wave ref. to templateNum, syncFile
						for ( count2 = 0; count2 < ( unsigned int ) wavePak->Count(); count2++ )
							if ( (*wavePak)[count2].templateNum == selectDeleteNum )
								if ( (*wavePak)[count2].fLock == INACTIVE )
									(*wavePak)[count2].templateNum = 0;
								else
									error++;
					}

					if ( !error )
					{
						(*templatePak)[count].name[0] = 0;
						(*templatePak)[count].syncFile[0] = 0;
					
						if ( (*templatePak)[count].graphFltCmd != NULL )
						{
							(*templatePak)[count].graphFltCmd->close();
							delete (*templatePak)[count].graphFltCmd;
						}
						
						if ( (*templatePak)[count].graphIntCmd != NULL )
						{
							(*templatePak)[count].graphIntCmd->close();
							delete (*templatePak)[count].graphIntCmd;
						}

						if ( (*templatePak)[count].pIntArray != NULL )
							delete (*templatePak)[count].pIntArray;

						if ( (*templatePak)[count].pFloatArray != NULL )
							delete (*templatePak)[count].pFloatArray;

						templatePak->RemoveItemAt ( count );
					}

					count = templatePak->Count();; 	// break out of loop
				}
			}

			// reallocate id numbers to be in sequence
			for ( count = 0 ; count < ( unsigned int ) templatePak->Count(); count++ )
			{
				// check for linked wave ref. to templateNum, syncFile
				for ( count2 = 0; count2 < ( unsigned int ) wavePak->Count(); count2++ )
					if ( (*wavePak)[count2].templateNum == (*templatePak)[count].templateNum )
						(*wavePak)[count2].templateNum = count + 1;

				(*templatePak)[count].templateNum = count + 1;

				if ( (*templatePak)[count].graphIntCmd != NULL )
				{
					(*templatePak)[count].graphNum = count + 1;	// reset in wavePak
					(*templatePak)[count].graphIntCmd->graphNum ( count + 1 );	// re-sync graph id to wave
				}

				if ( (*templatePak)[count].graphFltCmd != NULL )
				{
					(*templatePak)[count].graphNum = count + 1;	// reset in wavePak
					(*templatePak)[count].graphFltCmd->graphNum ( count + 1 );	// re-sync graph id to wave
				}
			}
		}

		break;
	}
}
