/*
    EditHelpCmd.cc

    Will Eighteen, 2000, 2001: will@opaline.demon.co.uk
*/

#include "EditHelpCmd.h"
#include "Application.h"
#include <TextEdit.h>


EditHelpCmd::EditHelpCmd ( const char *name, int active ) :  NoUndoCmd ( name, active )
{
	_editRealised = FALSE;
}


EditHelpCmd::~EditHelpCmd()
{
	// empty
}


void EditHelpCmd::doCommand()
{
	if (_editRealised == TRUE )
		return;

	_edit = new TextEdit ( ( char * ) name(), TRUE, &EditHelpCmd::okCallback, &EditHelpCmd::cancelCallback, ( void * ) this );	
	_edit->setRows ( 19 );
	_edit->setColumns ( 70 );
	_editRealised = TRUE;
	_edit->execute();
	_edit->setText ( "Menu button help follows the application button \
layout.\n\n\nEdit\nThe edit menu allows various operations to be performed on \
files. These are direct actions on the files. Views (if open) merely reflect \
the changes made to the file data. There is no undo, so it is recommended \
that operations are performed on copies of the files to avoid destruction of \
original data. Due to the direct-to-disk operation, there is no 'save'.\n\n\n\
Edit / Copy\nPops up a menu of source waves from which an item can be selected. \
The selected wave file must have a region selected in it for the copy \
operation to succeed. On activation, this menu is replaced by one containing \
a list of target waves. On selecting from this menu, the selected region will \
be copied to the position indicated by the start selection pointer in the \
target wave.\n\n\nEdit / Cut\nPops up a menu of wave files. Providing there has \
been a selection made in the wave selected, that region will be cut on button \
activation.\n\n\nEdit / <Shift\\scale>\nThis causes a dialogue box to appear. Either \
a wave or template may be selected as the target for the operation . The view \
of the relevant wave or template must be realised or an error message will be \
displayed. This is a safety feature so that operations cannot be performed on \
a file whilst is invisible. Activation of the relevant wave or template button \
produces a popup list of loaded files of that type. On activation, the name of \
the selected file is listed as the target at the top of the dialogue box.\n\
The text entry fields 'Shift' and 'Scale' are selected by left-clicking with \
the mouse pointer in the appropriate box. Values entered into these fields \
must be confirmed by pressing the 'return' key (the cursor will jump to the \
left-hand side of the box). Failure to confirm a value in a field will result \
in the previously-confirmed value being used.\nThe values for shift and scale \
operate as <datum value)*scale+shift.\nNote that for these operations to \
succeed there must be a selected region in the relevant file.\nThe actual \
operation to be performed is activated by pressing 'Execute' once the target \
shift and scale have been set.\n\n\nEdit / Shape template\nActivates a dialogue \
box allowing various predefined shapes to be applied to a selected region of \
a template. The 'Template' button pops up a list of loaded template files.\n\
For operations to succeed, the template view must be realised.\nText field \
entries must be confirmed with 'return' once a value has been entered (the \
cursor will move to the start of the text field. Once the data has been \
specified, activating the appropriate function button produces the required \
modification to the template data.\n\nLine: produces a line from \
initial to final values.\n\nSine: produces a sine wave with the specified \
number of repeats. The offset value is used to move the start point along the \
sine, so entering a value of pi/2 (approx. 1.5708) causes a cosine to be \
produced.\n\nTan: produces a tangent curve\n\nExp: produces an exponential \
curve\n\nLog: produces a logarithmic curve.\n\nNote: the default initial \
and final values can produce results outside the range of the view window, \
in which case it will be necessary to change the scale in the latter to \
render the curve visible. It should also be noted that these values change \
depending on \
whether an integer or floating-point template is used. It is also important \
to note that the range of values produced by these functions are dependent on \
the maximum and minimum values for integer and floating-point range specified \
in 'Options / Display / <INT\\FP> range'.\n\nSmooth: instead of specifying a shape \
to be applied to a template, this evens out irregularities in shape by an \
averaging process; it is most useful applied to templates hand-drawn with the \
mouse. The most useful values are those set as defaults (Points 3, Weighting \
2).The effect works best on sections of curve where the rate of change \
doesn't change sign, i.e. where the curve is either increasing or decreasing. \
\n\n\nEdit / Convert\nThis pops up a menu of file type converters. All work in \
the same manner. Selection of a converter type causes a file requestor to pop \
up with which the file to convert can be selected. Excepting the first item \
'Split stereo', this is followed by the appearance of a second file requestor \
into which the name of the converted file can be entered.\nNote that the \
values in the output file are modified by the values contained in the \
'Options / Convert / <type>' fields, which have the form of data pairs offset, \
scale.\n\nEdit / Convert / Split stereo\nSelection of a file in the file \
requestor automatically causes two mono output files to be written containing \
the two channels' data. These have identifiers for the left and right \
channels inserted in the source file name 'chL' and 'chR' so that 'name.wav' \
is converted into 'name.chL.wav' and 'name.chR.wav\n\nEdit / Convert / Wave to \
Raw\nThis simply strips the header from a mono wave file, leaving only the \
raw data; this is useful for import/export of files with some other programs, \
e.g. SLab, which latter operates rather better with raw data than .wav files.\n\n\
Edit / Convert / Raw to Wave\nOperates in the reverse fashion, applying a header \
defined by 'Options / File / Wave parameters' (the data size field, zero by default, \
is given the correct value once conversion is complete).\n\nEdit / Convert / \
Int to Wave\nConverts a file in integer format to a wave format file (as \
specified by Options / Wave/ Wave parameters).\n\nEdit / Convert / Wave \
to Int\nConverts a wave file to a file of integers. This allows use of wave \
files as templates. If a floating-point template is required, then the 'Int \
to Float' converter can be applied to the generated integer file. Note \
that during conversion the values of offset and scale defined in 'Options / \
Convert / <type> to Int' are applied: int_value+offset*scale. Note that the \
offset is applied first, then the scaling.\n\n\Edit / Convert / Int to Float\n\
Converts a file of integers into floating-point format. Again, the output \
file values are modified by the addition of an offset and subsequent scaling \
as above via 'Options / Convert / <type> to Float'.\n\nEdit / Convert / Float \
to Int\nConverts floating-point format number files to integer format. Note \
that the float value is rounded before conversion, so only the integer part \
is preserved. Scaling and offset apply.\n\nEdit / Convert / Ascii to \
Float\nConverts a file written in ascii (text) format to floating-point \
representation. The floating-point output is modified from the text input \
data by the formula ascii_value+offset*scale. The offset is applied to the \
number represented as text and it is then scaled. The offset and scale are \
set by 'Options / Convert / <type> to Float' and the floating-point precision \
by 'Options / Display / FP precision'.\n\nEdit / Convert / Float to Ascii\nThis \
converts a file of floating-point numbers into ascii (text) \
format. The representation of the floating point output is controlled by \
'Options / Display / FP precision', which has the default value 4, indicating \
that output files are written to 4 d.p. A value of zero simply causes the \
integer part of the number to be written, useful for ascii representation of \
integer files (convert to float then to ascii with FP precision 0).\n\n\n\
Edit / Convert / Csound\nThis gives access to a list of Csound file type \
converters. Currently only interconversion of the heterodyne analysis format \
and ascii is supported (the hetro / adsyn opcode pair). Selection of a converter \
from the menu causes source and target file browsers to appear in \
succession.\n\nEdit / Convert / Csound / Ascii to Hetro\nThis converts a file \
written in ascii (text) format into the machine-readable format used by the \
Csound 'adsyn' opcode.\nNote that the following conventions are implemented \
in the ascii file format:\n\n1: The first number in the file must be the \
the number of amplitude/frequency data track pairs (1 pair for each \
harmonic)\n\n2: sets of pairs of \
time/amplitude and time/frequency data follow. Each data pair is on a single \
line with a space as separator.\n\n3: time/amplitude data pairs must be \
preceded by '; amplitude breakpoint set' on a line before the data pairs; \
time/frequency data pairs must be preceded by '; frequency breakpoint set', \
again on a line preceding the data pairs.\n\n4: Data pair sets in either the \
amplitude or frequency domain must be followed by a line containing \
'; end track'.\n\nAny comment line may be inserted in the ascii file \
provided it starts with a semi-colon; this causes that line to be ignored, \
excepting the special commented lines given above which cause numeric \
control values to be written to the output file.\n\nEdit / Convert / \
Csound / Hetro to Ascii\nThis converter takes the output file produced by the \
'hetro' analysis opcode and converts it to an ascii (text) equivalent \
This makes the file easy to interpret and modify, since \
the text file can be edited in any editor or word processor. The text file \
can then be turned back into machine-readable form by the 'Ascii to Hetro' \
converter.");

	_edit->setEditStatus ( False );
}


void EditHelpCmd::okCallback ( XtPointer clientData, char * text )
{
	EditHelpCmd *obj = ( EditHelpCmd * ) clientData;

	obj->ok( clientData );	// ok mapped to 'enter'
}


void EditHelpCmd::cancelCallback ( XtPointer clientData )
{
	EditHelpCmd *obj = ( EditHelpCmd * ) clientData;

	obj->ok( clientData );	// call ok to delete object
}


void EditHelpCmd::ok ( void *clientData )
{
	EditHelpCmd *obj = ( EditHelpCmd * ) clientData;

	if (obj->_editRealised == TRUE )
	{
		obj->_editRealised = FALSE;
		delete obj->_edit;
	}
}
