/*
    WaveDataCmd.cc

    Will Eighteen, 2005, 2006: will@opaline.demon.co.uk
*/

#include "WaveDataCmd.h"
#include "Application.h"
#include <string.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/TextF.h>
#include <Xm/PushB.h>
#include <Xm/Frame.h>
#include <MessageDialogManager.h>
#include "AppWindow.h"
#include <Xm/Protocols.h>


WaveDataCmd::WaveDataCmd ( char *name, int active, WaveDataCallback callback,
							Wave *wave, int *data, int numDigits, long _dataOffset,
							long numDataPoints, void *clientData ) : NoUndoCmd ( name, active )
{
	_callback = callback;
	_numDigits = numDigits;
	_data = data;
	_clientData = clientData;
	_sampleRate = wave->sampleRate();
	 // -ve field values force update on first call to change field values
	_field1Value = _field2Value = _field3Value = -1;
	_field4Value = _field5Value = -1;
	_field6Value = -1;
	_numData = numDataPoints;	// current no, _data points in array
	_maxData = wave->totalSamples();
	_waveDataNumber = wave->waveNum();
	_startData = _dataOffset;
	_minY = mixwaveResources->minY();
	_maxY = mixwaveResources->maxY();
	_callbackType = 0;		// set by cancel
	strncpy ( _applicationName, ( char * ) application->applicationClass(), sizeof ( _applicationName ) );
	strncat ( _applicationName, "DataViewer", sizeof ( _applicationName ) - strlen ( _applicationName ) - 1 );
	// or the popup inherits the size of the top-level window
	strncpy ( _name, name, sizeof ( _name ) );
	_name [ sizeof ( _name ) - 1 ] = 0;
	_fpDigits = 5;	// single-sample resolution at 44.1 kHz
	_utils = new MixwaveUtils;
}


WaveDataCmd::~WaveDataCmd()
{
	// empty
}


void WaveDataCmd::doCommand()
{
	int n;
	Arg args[5];
	Atom WM_DELETE_WINDOW;

	_popup = XtCreatePopupShell ( _applicationName, transientShellWidgetClass,
									application->baseWidget(), NULL, 0 );

	_rc0 = XtVaCreateManagedWidget ( "RowColumn0", xmRowColumnWidgetClass,
			_popup,
			XmNorientation, XmVERTICAL,
			XmNentryAlignment, XmALIGNMENT_CENTER,
			XmNnumColumns, 2, NULL );

	// use the supplied class instance name for the label widget text
	_label = XtCreateManagedWidget ( _name, xmLabelWidgetClass, _rc0, NULL, 0 );

	_frame = XtVaCreateManagedWidget ( "frame", xmFrameWidgetClass, _rc0, NULL );

	_rca = XtVaCreateManagedWidget ( "RowColumnA", xmRowColumnWidgetClass,
			_frame,
			XmNentryAlignment, XmALIGNMENT_CENTER,
			XmNorientation, XmHORIZONTAL,
			XmNpacking, XmPACK_COLUMN,
	 		XmNnumColumns, 2, NULL );

	n = 0;

	XtSetArg ( args[n], XmNcolumns, _numDigits ); n++;
	XtSetArg ( args[n], XmNeditable, TRUE ); n++;
	XtSetArg ( args[n], XmNcursorPositionVisible, TRUE ); n++;
	XtSetArg ( args[n], XmNmaxLength, _numDigits ); n++;

	_label1 = XtCreateManagedWidget ( "label1", xmLabelWidgetClass, _rca, NULL, 0 );
	_label2 = XtCreateManagedWidget ( "label2", xmLabelWidgetClass, _rca, NULL, 0 );
	_label3 = XtCreateManagedWidget ( "label3", xmLabelWidgetClass, _rca, NULL, 0 );
	_label4 = XtCreateManagedWidget ( "label4", xmLabelWidgetClass, _rca, NULL, 0 );
	_label5 = XtCreateManagedWidget ( "label5", xmLabelWidgetClass, _rca, NULL, 0 );
	_label6 = XtCreateManagedWidget ( "label6", xmLabelWidgetClass, _rca, NULL, 0 );
	_label7 = XtCreateManagedWidget ( "label7", xmLabelWidgetClass, _rca, NULL, 0 );

	_field1 = XtCreateManagedWidget ( "field1", xmTextFieldWidgetClass, _rca, args, n );
	_field2 = XtCreateManagedWidget ( "field2", xmTextFieldWidgetClass, _rca, args, n );
	_field3 = XtCreateManagedWidget ( "field3", xmTextFieldWidgetClass, _rca, args, n );
	_field4 = XtCreateManagedWidget ( "field4", xmTextFieldWidgetClass, _rca, args, n );
	_field5 = XtCreateManagedWidget ( "field5", xmTextFieldWidgetClass, _rca, args, n );
	_field6 = XtCreateManagedWidget ( "field6", xmTextFieldWidgetClass, _rca, args, n );
	_field7 = XtCreateManagedWidget ( "field7", xmTextFieldWidgetClass, _rca, args, n );

	// Add callbacks for _data retrieval from parent
	XtAddCallback ( _field1, XmNactivateCallback, &WaveDataCmd::reloadCallback, ( XtPointer ) this );
	XtAddCallback ( _field2, XmNactivateCallback, &WaveDataCmd::reloadCallback, ( XtPointer ) this );
	XtAddCallback ( _field3, XmNactivateCallback, &WaveDataCmd::reloadCallback, ( XtPointer ) this );
	XtAddCallback ( _field4, XmNactivateCallback, &WaveDataCmd::reloadCallback, ( XtPointer ) this );
	XtAddCallback ( _field5, XmNactivateCallback, &WaveDataCmd::reloadCallback, ( XtPointer ) this );
	XtAddCallback ( _field7, XmNactivateCallback, &WaveDataCmd::reloadCallback, ( XtPointer ) this );

	// add callbacks for changed values
	XtAddCallback ( _field1, XmNvalueChangedCallback,
					&WaveDataCmd::field1ChangedCallback, ( XtPointer ) this );
	XtAddCallback ( _field2, XmNvalueChangedCallback,
					&WaveDataCmd::field2ChangedCallback, ( XtPointer ) this );
	XtAddCallback ( _field3, XmNvalueChangedCallback,
					&WaveDataCmd::field3ChangedCallback, ( XtPointer ) this );
	XtAddCallback ( _field4, XmNvalueChangedCallback,
					&WaveDataCmd::field4ChangedCallback, ( XtPointer ) this );
	XtAddCallback ( _field5, XmNvalueChangedCallback,
					&WaveDataCmd::field5ChangedCallback, ( XtPointer ) this );

	// add callbacks for _data entry in text widgets
	XtAddCallback ( _field1, XmNactivateCallback,
					&WaveDataCmd::field1EntryCallback, ( XtPointer )  this );
	XtAddCallback ( _field2, XmNactivateCallback,
					&WaveDataCmd::field2EntryCallback, ( XtPointer )  this );
	XtAddCallback ( _field3, XmNactivateCallback,
					&WaveDataCmd::field3EntryCallback, ( XtPointer )  this );
	XtAddCallback ( _field4, XmNactivateCallback,
					&WaveDataCmd::field4EntryCallback, ( XtPointer )  this );
	XtAddCallback ( _field5, XmNactivateCallback,
					&WaveDataCmd::field5EntryCallback, ( XtPointer )  this );
	XtAddCallback ( _field6, XmNactivateCallback,
					&WaveDataCmd::field6EntryCallback, ( XtPointer )  this );

	// cater for WM 'X' pressed
	WM_DELETE_WINDOW = XmInternAtom ( XtDisplay ( _popup ), "WM_DELETE_WINDOW", False );
	XmAddWMProtocols ( _popup, &WM_DELETE_WINDOW, 1 );
	XmAddWMProtocolCallback ( _popup, WM_DELETE_WINDOW, clientMessageProc, ( XtPointer ) this );

	XtManageChild ( _popup );
}


void WaveDataCmd::clientMessageProc ( Widget, XtPointer clientData, XtPointer cbs)
{
	WaveDataCmd *obj = ( WaveDataCmd * ) clientData;

	obj->clientMessage( ( XmAnyCallbackStruct * ) cbs );
}


void WaveDataCmd::clientMessage( XmAnyCallbackStruct *cbs )
{
	if ( cbs->reason != XmCR_CANCEL && cbs->reason != XmCR_OK )
	{
		// if not 'ok' or 'cancel', must be 'quit'
		(*wavePak)[_waveDataNumber-1].waveDataCmd = NULL;
		cancel();
	}
}


void WaveDataCmd::setFields ( long start, long end, long selStart, long selEnd,
								long xCurrent, long yCurrent, long maxDataPoints )
{
	long region = selEnd - selStart;
	changeField1 ( start );
	changeField2 ( end );
	changeField3 ( selStart );
	changeField4 ( selEnd );
	changeField5 ( xCurrent );
	changeField6 ( yCurrent );
	changeField7 ( region );
	_maxData = maxDataPoints;
	_oldStart = start;
	_oldEnd = end;
}


void WaveDataCmd::setDatumValueBounds ( long minY, long maxY )
{
	char buf[2*_numDigits+1];
	
	snprintf ( buf, sizeof ( buf ), "%ld", minY );
	buf [ sizeof ( buf ) - 1 ] = 0;
	
	if ( strlen ( buf ) <= _numDigits )
		_minY = minY;
		
	snprintf ( buf, sizeof ( buf ), "%ld", maxY );
	buf [ sizeof ( buf ) - 1 ] = 0;
	
	if ( strlen ( buf ) <= _numDigits )
		_maxY = maxY;
}


void WaveDataCmd::reloadCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	WaveDataCmd *obj = ( WaveDataCmd * ) clientData;

	obj->reload();
}


void WaveDataCmd::reload()
{
	if ( (*wavePak)[_waveDataNumber - 1].fLock == ACTIVE )
		return;

	Boolean reload;
	long f6Val;

	if ( _field6Value < 0 )
		f6Val =  ( long ) ( ( ( _field6Value * ( float ) -_minY ) / 100 ) - 0.5 );
	else
		f6Val =  ( long ) ( ( ( _field6Value * ( float ) _maxY ) / 100 ) + 0.5 );

	if ( _callback )
	{
		if ( _field2Value < _field1Value )
			_field2Value = _field1Value;
			
		if ( _field4Value < _field3Value )
			_field4Value = _field3Value;

		if ( ( _field1Value < _oldStart ) || ( _field3Value < _oldStart )
				|| ( _field2Value > _oldEnd ) || ( _field4Value > _oldEnd ) )
			reload = True;
		else
			reload = False;

		_callback ( this, _clientData, _field1Value, _field2Value, _field3Value,
					_field4Value, _field5Value, f6Val, reload, _waveDataNumber, _callbackType );
	}
}


void WaveDataCmd::datumChangeOkCallback ( XtPointer clientData )
{
	WaveDataCmd* obj = ( WaveDataCmd * ) clientData;

	obj->datumChangeOk ( obj->_field5Value );
}


void WaveDataCmd::datumChangeCancelCallback ( XtPointer clientData )
{
	WaveDataCmd* obj = ( WaveDataCmd * ) clientData;

	// existing value is ok, change text field back to _field6Value
	obj->changeField6 ( ( long ) obj->_field6Value );
}


void WaveDataCmd::field1ChangedCallback ( Widget, XtPointer clientData, XtPointer )
{
        WaveDataCmd *obj = ( WaveDataCmd * ) clientData;
	char *text;

	text = XmTextFieldGetString ( obj->_field1 );

	if ( strlen ( text ) > 0 )
	      	obj->field1Changed ( text );
}


void WaveDataCmd::field2ChangedCallback ( Widget, XtPointer clientData, XtPointer )
{
	WaveDataCmd *obj = ( WaveDataCmd * ) clientData;
	char *text;
		
	text = XmTextFieldGetString ( obj->_field2 );
		
	if ( strlen ( text ) > 0 )
		obj->field2Changed ( text );
}


void WaveDataCmd::field3ChangedCallback ( Widget, XtPointer clientData, XtPointer )
{
        WaveDataCmd *obj = ( WaveDataCmd * ) clientData;
	char *text;

	text = XmTextFieldGetString ( obj->_field3 );

	if ( strlen ( text ) > 0 )
       		obj->field3Changed ( text );
}


void WaveDataCmd::field4ChangedCallback ( Widget, XtPointer clientData, XtPointer )
{
        WaveDataCmd *obj = ( WaveDataCmd * ) clientData;
	char *text;

	text = XmTextFieldGetString ( obj->_field4 );

	if ( strlen ( text ) > 0 )
	       	obj->field4Changed ( text );
}


void WaveDataCmd::field5ChangedCallback ( Widget, XtPointer clientData, XtPointer )
{
        WaveDataCmd *obj = ( WaveDataCmd * ) clientData;
	char *text;
		
	text = XmTextFieldGetString ( obj->_field5 );

	if ( strlen ( text ) > 0 )
      		obj->field5Changed ( text );
}


void WaveDataCmd::field1EntryCallback ( Widget, XtPointer clientData, XtPointer )
{
	WaveDataCmd* obj = ( WaveDataCmd * ) clientData;
	char text[obj->_numDigits+1];

	snprintf ( text, sizeof ( text ), "%s", XmTextFieldGetString ( obj->_field1 ) );
	text [ sizeof ( text ) - 1 ] = 0;

	if ( strlen ( text ) == 0 )
		obj->changeField1 ( 0 );

	obj->field1Entry ( obj->_field1Value );
}


void WaveDataCmd::field2EntryCallback ( Widget, XtPointer clientData, XtPointer )
{
	WaveDataCmd* obj = ( WaveDataCmd * ) clientData;
	char text[obj->_numDigits+1];

	snprintf ( text, sizeof( text ), "%s", XmTextFieldGetString ( obj->_field2 ) );
	text [ sizeof ( text ) - 1 ] = 0;

	if ( strlen ( text ) == 0 )
		obj->changeField2 ( 0 );

	obj->field2Entry ( obj->_field2Value );
}


void WaveDataCmd::field3EntryCallback ( Widget, XtPointer clientData, XtPointer )
{
	WaveDataCmd* obj = ( WaveDataCmd * ) clientData;
	char text[obj->_numDigits+1];

	snprintf ( text, sizeof ( text ), "%s", XmTextFieldGetString ( obj->_field3 ) );
	text [ sizeof ( text ) - 1 ] = 0;

	if ( strlen ( text ) == 0 )	// if not, the value already corresponds
		obj->changeField3 ( obj->_field1Value );	// start sel is set to start display

	obj->field3Entry ( obj->_field3Value );
}


void WaveDataCmd::field4EntryCallback ( Widget, XtPointer clientData, XtPointer )
{
	WaveDataCmd* obj = ( WaveDataCmd * ) clientData;
	char text[obj->_numDigits+1];

	snprintf ( text, sizeof ( text ),"%s", XmTextFieldGetString ( obj->_field4 ) );
	text [ sizeof ( text ) - 1 ] = 0;
	
	if ( strlen ( text ) == 0 )
		obj->changeField4 ( obj->_numData - 1 );	// set end sel to end display

	obj->field4Entry ( obj->_field4Value );
}


void WaveDataCmd::field5EntryCallback ( Widget, XtPointer clientData, XtPointer )
{
	WaveDataCmd* obj = ( WaveDataCmd * ) clientData;
	char text[obj->_numDigits+1];

	snprintf ( text, sizeof ( text ), "%s", XmTextFieldGetString ( obj->_field5 ) );
	text [ sizeof ( text ) - 1 ] = 0;

	if ( strlen ( text ) == 0 )
	{
		obj->changeField5 ( obj->_field1Value );
		return;	// we didn't specify a datum number
	}

	obj->field5Entry ( obj->_field5Value );
}


void WaveDataCmd::field6EntryCallback ( Widget, XtPointer clientData, XtPointer )
{
	WaveDataCmd* obj = ( WaveDataCmd * ) clientData;
	char msg[256];
	char title[128];
	char numText[obj->_numDigits+1];

	// this changes actual _data. post dialog with callback to change
	snprintf ( numText, sizeof ( numText ), "%s", XmTextFieldGetString ( obj->_field6 ) );
	numText [ sizeof ( numText ) - 1 ] = 0;
	strncpy ( title, obj->_name, sizeof ( title ) );
	title [ sizeof ( title ) - 1 ] = 0;
	snprintf ( msg, sizeof ( msg ), "%s\n\nReally alter percentage datum value from %f to %s ?", title, obj->_field6Value, numText );
	msg [ sizeof ( msg ) - 1 ] = 0;

	messageDialogManager->post ( msg, ( XtPointer ) clientData,
					&WaveDataCmd::datumChangeOkCallback,
					&WaveDataCmd::datumChangeCancelCallback );
}


void WaveDataCmd::field1Entry ( long value )
{
	if ( value > _maxData - 1 )
	{
		value = _maxData - 1;
		changeField1 (value );
	}

	if ( value >= _field2Value )
	{
		changeField2 ( value );	// results in y value error, datumNum wrong
		messageDialogManager->post ( "No data to display in range." );		
	}
}


void WaveDataCmd::field2Entry ( long value )
{
	if ( value > _maxData - 1 )
	{
		value = _maxData - 1;
		changeField2 ( value );
	}
		
	if ( value <= _field1Value )
	{
		changeField1 ( value );
		messageDialogManager->post ( "No data to display in range." );
	}
}


void WaveDataCmd::field3Entry ( long value )
{
	if ( value > _maxData - 1 )
	{
		value = _maxData - 1;
		changeField3 ( value );
	}

	if ( value >= _field4Value )
	{
		changeField4 ( value );
		messageDialogManager->post ( "No data to display in selection." );
	}
}


void WaveDataCmd::field4Entry ( long value )
{
	if ( value > _maxData - 1 )
	{
		value = _maxData - 1;
		changeField4 ( value );
	}

	if ( value <= _field3Value )
	{
		changeField3 ( value );
		messageDialogManager->post ( "No data to display in selection." );
	}
}


void WaveDataCmd::field5Entry ( long value )
{
	if ( value < _startData )
		changeField5 ( _startData );
			
	if ( value >  ( _numData - 1 ) )
	{
		if ( value > ( _maxData - 1 ) )
		{
			value = _maxData - 1;	
			changeField5 ( value );
		}
	}
}

/* since these continually update the GraphCmd object, range and selection
   validation required here rather than in entry functions. */

void WaveDataCmd::field1Changed ( char *text )
{
	long value;

	if ( !_utils->checkTimeStringFormat ( text ) )	// no error returned
	{
		value = _utils->timeStringToSampleNum ( text, _sampleRate );

		if ( value > ( _maxData - 1 ) )
		{
			changeField1 ( _maxData - 1 );
			changeField2 ( _maxData - 1 );
		}
		else
			_field1Value = value;
	}
}


void WaveDataCmd::field2Changed ( char *text )
{	// 'e' is reserved for 'end file'
	long value;
	char c;

	c = text[0];

	if ( c == 'e' )
		changeField2 ( ( long ) _maxData - 1 );
	else
	{
		if ( !_utils->checkTimeStringFormat ( text ) )
		{
			value = _utils->timeStringToSampleNum ( text, _sampleRate );

			if ( value > ( _maxData - 1 ) )
				changeField2 ( _maxData - 1 );
			else
				_field2Value = value;
		}
	}
}


void WaveDataCmd::field3Changed ( char *text )
{
	long value;
	
	if ( !_utils->checkTimeStringFormat ( text ) )
	{
		value = _utils->timeStringToSampleNum ( text, _sampleRate );

		if ( value > ( _maxData - 1 ) )
		{
			changeField3 ( _maxData - 1 );
			changeField4 ( _maxData - 1 );
		}
		else
			_field3Value = value;
	}
}


void WaveDataCmd::field4Changed ( char *text )
{	// 'e' is reserved for 'end file'

	long value;
	char c;

	c = text[0];

	if ( c == 'e' )
		changeField4 ( ( long ) _maxData - 1 );
	else
	{
		if ( !_utils->checkTimeStringFormat ( text ) )
		{
			value = _utils->timeStringToSampleNum ( text, _sampleRate );

			if ( value > ( _maxData - 1 ) )
				changeField4 ( _maxData - 1 );
			else
				_field4Value = value;
		}
	}
}


void WaveDataCmd::field5Changed ( char *text )
{	// change only allowed inside existing view range
	long value;

	if ( !_utils->checkTimeStringFormat ( text ) )
	{
		value = _utils->timeStringToSampleNum ( text, _sampleRate );
		_field5Value = value;

		if ( value < _field1Value )
			_field5Value = _field1Value;	// keep callback value within array bounds
		
		if ( value > _field2Value )
			_field5Value = _field2Value;
	}
}


void WaveDataCmd::datumChangeOk ( long datumNumber )
{
	char numText[_numDigits+1];
	float fValue;
	long lValue;

	snprintf ( numText, sizeof ( numText ), "%s", XmTextFieldGetString ( _field6 ) );
	numText [ sizeof ( numText ) - 1 ] = 0;
	sscanf ( numText, "%f", &fValue );

	// convert float % peak to actual datum value
	if ( fValue < 0.0 )
		lValue = ( long ) ( ( fValue / 100 ) * -_minY );
	else
		lValue = ( long ) ( ( fValue / 100 ) * _maxY );

	// check datum value is within allowed limits
	if ( ( lValue >= _minY ) && ( lValue <= _maxY ) )
	{
		_data[datumNumber-_startData] = lValue;
		changeField6 ( lValue );
		reload();		
	}
}


/* changeField: methods to set text box string from numeric value */

void WaveDataCmd::changeField1 ( long value )
{
	if ( _field1Value == value )
		return;

	char text[_numDigits+1];

	strncpy ( text, _utils->sampleNumToTimeString ( value, _sampleRate, _numDigits, _fpDigits ), sizeof ( text ) );
	text [ sizeof ( text ) - 1 ] = 0;
	_field1Value = value;
	XmTextFieldSetString ( _field1, text );
}


void WaveDataCmd::changeField2 ( long value )
{
	if ( _field2Value == value )
		return;

	char text[_numDigits+1];
	
	strncpy ( text, _utils->sampleNumToTimeString ( value, _sampleRate, _numDigits, _fpDigits ), sizeof ( text ) );
	text [ sizeof ( text ) - 1 ] = 0;
	_field2Value = value;
	XmTextFieldSetString ( _field2, text );
}


void WaveDataCmd::changeField3 ( long value )
{
	if ( _field3Value == value )
		return;

	// no access to parent obj selection locks...
	char text[_numDigits+1];
	
	strncpy ( text, _utils->sampleNumToTimeString ( value, _sampleRate, _numDigits, _fpDigits ), sizeof ( text ) );
	text [ sizeof ( text ) - 1 ] = 0;
	_field3Value = value;
	XmTextFieldSetString ( _field3, text );
}


void WaveDataCmd::changeField4 ( long value )
{
	if ( _field4Value == value )
		return;

	char text[_numDigits+1];
	
	strncpy ( text, _utils->sampleNumToTimeString ( value, _sampleRate, _numDigits, _fpDigits ), sizeof ( text ) );
	text [ sizeof ( text ) - 1 ] = 0;
	_field4Value = value;
	XmTextFieldSetString ( _field4, text );
}


void WaveDataCmd::changeField5 ( long value )
{
	if ( _field5Value == value )
		return;

	char text[_numDigits+1];

	strncpy ( text, _utils->sampleNumToTimeString ( value, _sampleRate, _numDigits, _fpDigits ), sizeof ( text ) );
	text [ sizeof ( text ) - 1 ] = 0;
	_field5Value = value;
	XmTextFieldSetString ( _field5, text );
}


void WaveDataCmd::changeField6 ( long value )
{
	char text[_numDigits+1];
	char formatString[2*_numDigits+1];

	snprintf ( formatString, sizeof ( formatString ), "%s%d%s", "%.", _fpDigits, "f" );
	formatString [ sizeof ( formatString ) - 1 ] = 0;
	
	if ( value < 0 )
		_field6Value =  ( float ) value / ( ( float ) -_minY ) * 100;
	else
		_field6Value =  ( float ) value / ( ( float ) _maxY ) * 100;

	snprintf ( text, sizeof ( text ), formatString, _field6Value );
	text [ sizeof ( text ) - 1 ] = 0;

	XmTextFieldSetString ( _field6, text );
}


void WaveDataCmd::changeField7 ( long value )
{
	char text[_numDigits+1];
	
	if ( value < 0 )
		value = 0;	// don't adjust field on r-to-l drag
	strncpy ( text, _utils->sampleNumToTimeString ( value, _sampleRate, _numDigits, _fpDigits ), sizeof ( text ) );
	text [ sizeof ( text ) - 1 ] = 0;
	XmTextFieldSetString ( _field7, text );
}


void WaveDataCmd::cancel()
{
	delete _utils;
	XtPopdown ( _popup );
	XtDestroyWidget ( _popup );
}


void WaveDataCmd::checkSelection()
{
	if ( _field3Value > _field4Value )	// we dragged right-to-left, swap
	{	
		long temp = _field4Value;
		changeField4 ( _field3Value );
		changeField3 ( temp );
	}
}


long WaveDataCmd::datumValue ( long datumNum )
{	// relative datum num in array
	long value;

	value = ( long ) _data [ datumNum ];
	return value;
}


// for external use if underlying wave id changes after realisation
void WaveDataCmd::waveDataNum ( int num )
{
	if ( num > 0 )
		_waveDataNumber = num;
}
