/*
	TextEdit.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk             


	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	See COPYING for full details.
*/

#include "TextEdit.h"
#include "Application.h"
#include <stdlib.h>
#include <string.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Protocols.h>
#include <stdio.h>

TextEdit::TextEdit ( char *name, int active, TextCallback callback,
						TextCancelledCallback cancelCallback, void *clientData ) : NoUndoCmd ( name, active )
{
	_callback = callback;
	_cancelCallback = cancelCallback;
	_clientData = clientData;
	_helpText = NULL;
	_presetText = 0;
	_columns = 0;
	_rows = 0;
	_data = NULL;
	_copy = NULL;
	
	strncpy ( _applicationName, ( char * ) application->applicationClass(), sizeof ( _applicationName ) );
	_applicationName [ sizeof ( _applicationName ) - 1 ] = 0;
	strncat ( _applicationName, " Text Editor", sizeof ( _applicationName ) - strlen ( _applicationName ) - 1 );
 	// or the popup inherits the size of the top-level window
	strncpy ( _name, name, sizeof ( _name ) );
	_name [ sizeof ( _name ) - 1 ] = 0;
}


void TextEdit::doCommand()
{
	int n;
	Arg args[6];
	Atom WM_DELETE_WINDOW;

	_popup = XtCreatePopupShell ( _applicationName, transientShellWidgetClass,
									application->baseWidget(), NULL, 0 );

	_rc1 = XtVaCreateManagedWidget ( "RowColumn1", xmRowColumnWidgetClass, _popup,
				XmNorientation, XmVERTICAL,
				XmNpacking, XmPACK_TIGHT,
				XmNnumColumns, 1, NULL );

	// use the supplied class instance name for the label widget text
	_label = XtCreateManagedWidget ( _name, xmLabelWidgetClass, _rc1, NULL, 0 );

	n = 0;
	
	XtSetArg ( args[n], XmNeditMode, XmMULTI_LINE_EDIT ); n++;
	XtSetArg ( args[n], XmNwordWrap, TRUE ); n++;
	XtSetArg ( args[n], XmNscrollHorizontal, FALSE ); n++;
	XtSetArg ( args[n], XmNeditable , False ); n++;

	if ( _columns )
		XtSetArg ( args[n], XmNcolumns, _columns ); n++;
		
	if ( _rows )
		XtSetArg ( args[n], XmNrows, _rows ); n++;

	_text = XmCreateScrolledText ( _rc1, "text", args, n );

	// now create another rowcol widget on the first
	_rc2 = XtVaCreateManagedWidget ( "RowColumn2", xmRowColumnWidgetClass, _rc1,
				XmNorientation, XmVERTICAL,
				XmNpacking, XmPACK_COLUMN,
				XmNnumColumns, 5, NULL );
				
	_ok = XtCreateManagedWidget ( "OK", xmPushButtonWidgetClass, _rc2, NULL, 0 );
	XtAddCallback ( _ok, XmNactivateCallback, &TextEdit::okCallback, ( XtPointer ) this );

	_cancel = XtCreateManagedWidget ( "Cancel", xmPushButtonWidgetClass, _rc2, NULL, 0 );
	XtAddCallback ( _cancel, XmNactivateCallback, &TextEdit::cancelCallback, ( XtPointer ) this );

	// if there's no help we don't need the buttons
	if ( _helpText )
	{
		_clear = XtCreateManagedWidget ( "Clear", xmPushButtonWidgetClass, _rc2, NULL, 0 );
		XtAddCallback ( _clear, XmNactivateCallback, &TextEdit::clearCallback, ( XtPointer ) this );

		_refresh = XtCreateManagedWidget ( "Refresh", xmPushButtonWidgetClass, _rc2, NULL, 0 );
		XtAddCallback ( _refresh, XmNactivateCallback, &TextEdit::refreshCallback, ( XtPointer ) this );

		_help = XtCreateManagedWidget ( "Help", xmPushButtonWidgetClass, _rc2, NULL, 0 );
		XtAddCallback ( _help, XmNactivateCallback, &TextEdit::helpCallback, ( XtPointer ) this );
	}

	// cater for WM 'X' pressed
	WM_DELETE_WINDOW = XmInternAtom ( XtDisplay ( _popup ), "WM_DELETE_WINDOW", False );
	XmAddWMProtocols ( _popup, &WM_DELETE_WINDOW, 1 );
 	XmAddWMProtocolCallback ( _popup, WM_DELETE_WINDOW, clientMessageProc, ( XtPointer ) this );

	XtPopup ( _popup, ( XtGrabKind ) XtGrabNone );
	XtManageChild ( _popup );
}


void TextEdit::clientMessageProc ( Widget w, XtPointer clientData, XtPointer cbs )
{
	TextEdit *obj = ( TextEdit * ) clientData;
	obj->clientMessage ( ( XmAnyCallbackStruct * ) cbs );

		XtDestroyWidget ( w );
}


void TextEdit::clientMessage ( XmAnyCallbackStruct *cbs )
{
	if ( cbs->reason != XmCR_CANCEL && cbs->reason != XmCR_OK )
		cancel();
}


void TextEdit::okCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	TextEdit *obj = ( TextEdit * ) clientData;

	if ( !obj->_presetText )
		obj->_data = XmTextGetString ( obj->_text );
	else
	{
		obj->_data = ( char * ) obj->_copy;
		obj->_presetText = 0;
	}

	obj->ok( ( char * ) obj->_data );	// ok mapped to 'enter'

	XtDestroyWidget ( w );
}


void TextEdit::ok ( char *text )
{
	if ( _callback )
		_callback ( _clientData, ( char * ) text );

	XtPopdown ( _popup );
}


void TextEdit::clearCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	TextEdit *obj = ( TextEdit * ) clientData;

	if ( !obj->_presetText )
		obj->_copy = XmTextGetString ( obj->_text );

	obj->clear();
}


void TextEdit::clear()
{
	char buf[1];
	
	buf[0] = 0;
		
	XmTextSetString ( _text, buf );
	_presetText = 1;
}


void TextEdit::refreshCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	TextEdit *obj = ( TextEdit * ) clientData;
	
	if ( obj->_presetText )
	{
		obj->_data = obj->_copy;
		XmTextSetString ( obj->_text, ( char * ) obj->_data );
		obj->_presetText = 0;
	}
}


void TextEdit::helpCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	TextEdit *obj = ( TextEdit * ) clientData;

	obj->help( clientData );	// preset help text supplied on invocation
}


void TextEdit::help( void *clientData )
{
	TextEdit *obj = ( TextEdit * ) clientData;

	if ( !_presetText )
		obj->_copy = XmTextGetString(obj->_text);	// keep copy of text in case it was edited

	XmTextSetString ( obj->_text, ( char * ) obj->_helpText );
	_presetText = 1;
	XtVaSetValues ( obj->_help, XtNname, "Text", NULL );

	obj->_data = obj->_copy;	// restore original text
}


void TextEdit::cancelCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	TextEdit *obj = ( TextEdit * ) clientData;
	obj->cancel();
	XtDestroyWidget ( w );
}


void TextEdit::cancel()
{
	if ( _cancelCallback ) 
		_cancelCallback ( _clientData );
	
	XtPopdown ( _popup );
}


void TextEdit::setText ( char *text )
{
	_data = text;
	_copy = text;	// in case help/clear called
	XmTextSetString ( _text, ( char * ) _data );
}


void TextEdit::setColumns ( int columns )
{
	_columns = columns;
}


void TextEdit::setRows ( int rows )
{
	_rows = rows;
}


void TextEdit::setEditStatus ( Boolean status )
{
	XtVaSetValues ( _text, XmNeditable, status, NULL );
}


void TextEdit::close()
{
	cancel();
	XtDestroyWidget ( _popup );
}
